import daiquiri.core.env as env

OAI_SCHEMA = env.get('OAI_SCHEMA', 'oai_schema')

OAI_ADAPTER = 'daiquiri.oai.adapter.DefaultOaiAdapter'
OAI_METADATA_FORMATS = [
    {
        'prefix': 'oai_dc',
        'schema': 'http://www.openarchives.org/OAI/2.0/oai_dc.xsd',
        'namespace': 'http://www.openarchives.org/OAI/2.0/oai_dc/',
        'renderer_class': 'daiquiri.oai.renderers.DublincoreRenderer'
    },
    {
        'prefix': 'oai_datacite',
        'schema': 'http://schema.datacite.org/oai/oai-1.1/oai.xsd',
        'namespace': 'http://schema.datacite.org/oai/oai-1.1/',
        'renderer_class': 'daiquiri.oai.renderers.OaiDataciteRenderer'
    },
    {
        'prefix': 'datacite',
        'schema': 'http://schema.datacite.org/meta/nonexistant/nonexistant.xsd',
        'namespace': 'http://datacite.org/schema/nonexistant',
        'renderer_class': 'daiquiri.oai.renderers.DataciteRenderer'
    },
    {
        'prefix': 'ivo_vor',
        'schema': 'http://www.ivoa.net/xml/RegistryInterface/v1.0',
        'namespace': 'http://www.ivoa.net/xml/RegistryInterface/v1.0',
        'renderer_class': 'daiquiri.oai.renderers.VoresourceRenderer'
    }
]
OAI_SETS = True
OAI_DELETED_RECORD = 'transient'
OAI_PAGE_SIZE = 500



import subprocess

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('operation', choices=[
            'start',
            'stop',
            'stopwait',
            'kill',
            'restart'
        ])

    def handle(self, *args, **options):
        if not settings.CELERY_PIDFILE_PATH:
            raise CommandError('CELERY_PIDFILE_PATH is not set')

        queues = [{
            'node': '{}_{}'.format(settings.DAIQUIRI_APP, queue['key']),
            'queue': queue['key'],
            'concurrency': queue.get('concurrency', 1)
        } for queue in settings.QUEUES] + [{
            'node': '{}_query_{}'.format(settings.DAIQUIRI_APP, queue['key']),
            'queue': 'query_{}'.format(queue['key']),
            'concurrency': queue.get('concurrency', 1)
        } for queue in settings.QUERY_QUEUES]

        args = [settings.CELERY_BIN, '-A', 'config', 'multi', options['operation']]
        args += [queue['node'] for queue in queues]

        if options['operation'] in ['start', 'restart']:
            for queue in queues:
                args += [
                    '-Q:{}'.format(queue['node']), queue['queue'],
                    '-c:{}'.format(queue['node']), str(queue['concurrency'])
                ]

        args += [
            f'--pidfile={settings.CELERY_PIDFILE_PATH}/%n.pid',
            f'--loglevel={settings.CELERY_LOG_LEVEL}'
        ]
        if settings.CELERY_LOG_PATH:
            args += [
                f'--logfile={settings.CELERY_LOG_PATH}/%n.log'
            ]

        subprocess.call(args)



from django.conf import settings
from django.urls import reverse

from daiquiri.core.vo import get_curation

from .adapter import ConeSearchAdapter


def get_resource():
    return {
        'service': 'conesearch',
        'identifier': f'ivo://{settings.SITE_IDENTIFIER}/conesearch',
        'title': f'{settings.SITE_TITLE} Cone Search Service',
        'curation': get_curation(),
        'content': {
            'subjects': settings.CONESEARCH_SUBJECTS,
            'type': 'Catalog',
            'description': f'The Cone Search Service for {settings.SITE_IDENTIFIER}.',
            'referenceURL': settings.SITE_URL.rstrip('/')  # + reverse('conesearch:root')
        },
        'capabilities': get_capabilities(),
        'created': settings.SITE_CREATED,
        'updated': settings.SITE_UPDATED,
        'type': 'vs:CatalogService',
        'status': 'active'
    }


def get_availability():
    return {
        'available': 'true',
        'note': 'service is accepting queries'
    }


def get_capabilities():
    adapter = ConeSearchAdapter()

    return [
        {
            'id': 'ivo://ivoa.net/std/ConeSearch',
            'type': 'cs:ConeSearch',
            'interface': {
                'type': 'vs:ParamHTTP',
                'role': 'std',
                'access_url': {
                    'use': 'base',
                    'url': settings.SITE_URL.rstrip('/') + reverse('conesearch:search', args=[resource])
                },
                'query_types': ['GET'],
                'result_type': 'application/x-votable+xml',
                'params': [
                    {
                        'std': 'true',
                        'name': 'RA',
                        'description': 'Right Ascension (ICRS decimal)',
                        'ucd': 'pos.eq.ra',
                        'datatype': 'real'
                    },
                    {
                        'std': 'true',
                        'name': 'DEC',
                        'description': 'Declination (ICRS decimal)',
                        'ucd': 'pos.eq.dec',
                        'datatype': 'real'
                    },
                    {
                        'std': 'true',
                        'name': 'SR',
                        'description': 'Search radius',
                        'datatype': 'real'
                    },
                    {
                        'std': 'true',
                        'name': 'VERB',
                        'description': 'Exhaustiveness of column selection. VERB=1 only returns the most important ' \
                                       'columns, VERB=2 selects the columns deemed useful to the average user, ' \
                                       'VERB=3 returns a table with all available columns.',
                        'datatype': 'integer'
                    }
                ]
            },
            'max_sr': adapter.ranges['SR']['max'],
            'max_records': adapter.max_records,
            'verbosity': 'true',
            'test_query': {
                'ra': adapter.defaults.get('RA'),
                'dec': adapter.defaults.get('DEC'),
                'sr': adapter.defaults.get('SR')
            }
        } for resource in adapter.get_resources()
    ]



class QueryFormAdapter:

    def get_fields(self):
        raise NotImplementedError

    def get_query_language(self, data):
        raise NotImplementedError

    def get_query(self, data):
        raise NotImplementedError



# Generated by Django 2.1.3 on 2018-11-05 16:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_auth', '0011_meta'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='profile',
            options={'ordering': ('user__last_name', 'user__last_name', 'user__username'), 'verbose_name': 'Profile', 'verbose_name_plural': 'Profiles'},
        ),
    ]



from django.conf import settings
from django.contrib.auth.models import Group

from rest_framework import serializers

from daiquiri.core.serializers import JSONListField

from ..models import Column, Function, Schema, Table
from .validators import PersonListValidator


class GroupSerializer(serializers.ModelSerializer):

    class Meta:
        model = Group
        fields = ('id', 'name')


class FunctionSerializer(serializers.ModelSerializer):

    label = serializers.CharField(source='__str__', read_only=True)
    admin_url = serializers.CharField(read_only=True)

    class Meta:
        model = Function
        fields = '__all__'


class ColumnSerializer(serializers.ModelSerializer):

    label = serializers.CharField(source='__str__', read_only=True)
    width = serializers.IntegerField(source='get_width', read_only=True)
    admin_url = serializers.CharField(read_only=True)

    class Meta:
        model = Column

        # only show access_level, metadata_access_level, and groups when
        # settings.METADATA_COLUMN_PERMISSIONS is set
        if settings.METADATA_COLUMN_PERMISSIONS:
            fields = '__all__'
        else:
            fields = (
                'id',
                'label',
                'order',
                'name',
                'description',
                'unit',
                'ucd',
                'utype',
                'datatype',
                'arraysize',
                'index_for',
                'principal',
                'indexed',
                'std',
                'table',
                'admin_url',
                'width',
            )


class TableSerializer(serializers.ModelSerializer):

    label = serializers.CharField(source='__str__', read_only=True)

    related_identifiers = JSONListField(required=False)
    creators = JSONListField(required=False, validators=[PersonListValidator()])
    contributors = JSONListField(required=False, validators=[PersonListValidator()])
    license = serializers.ChoiceField(choices=settings.LICENSE_CHOICES, default='')
    admin_url = serializers.CharField(read_only=True)

    class Meta:
        model = Table
        fields = '__all__'


class SchemaSerializer(serializers.ModelSerializer):

    label = serializers.CharField(source='__str__', read_only=True)

    related_identifiers = JSONListField(required=False)
    creators = JSONListField(required=False, validators=[PersonListValidator()])
    contributors = JSONListField(required=False, validators=[PersonListValidator()])
    license = serializers.ChoiceField(choices=settings.LICENSE_CHOICES, default='', initial='')
    admin_url = serializers.CharField(read_only=True)

    class Meta:
        model = Schema
        fields = '__all__'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-05-29 14:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0014_blank_run_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='creation_time',
            field=models.DateTimeField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='job',
            name='phase',
            field=models.CharField(choices=[('PENDING', 'Pending'), ('QUEUED', 'Queued'), ('EXECUTING', 'Executing'), ('COMPLETED', 'Completed'), ('ERROR', 'Error'), ('ABORTED', 'Aborted'), ('UNKNOWN', 'Unknown'), ('HELD', 'Held'), ('SUSPENDED', 'Suspended'), ('ARCHIVED', 'Archived')], db_index=True, max_length=10),
        ),
        migrations.AlterField(
            model_name='job',
            name='run_id',
            field=models.CharField(blank=True, db_index=True, default='', max_length=64),
        ),
    ]



import re

from django.core import validators
from django.utils.deconstruct import deconstructible
from django.utils.translation import gettext_lazy as _

'''
The validator should be included by adding:

    ACCOUNT_USERNAME_VALIDATORS = 'daiquiri.auth.validators.username_validators'

to settings/base.py.
'''

@deconstructible
class DaiquiriUsernameValidator(validators.RegexValidator):
    '''
    This is almost identical to https://github.com/django/django/blob/main/django/core/validators.py
    '''
    regex = r'^[a-z0-9_]+\Z'
    message = _(
        'Enter a valid username. This value may contain only English lowercase letters, '
        'numbers, and underscores.'
    )
    flags = re.ASCII


username_validators = [DaiquiriUsernameValidator()]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-03-08 16:06
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DetailKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.SlugField()),
                ('label', models.CharField(max_length=256)),
                ('data_type', models.CharField(choices=[('text', 'Text'), ('textarea', 'Textarea'), ('checkbox', 'Checkbox'), ('radio', 'Radio button'), ('select', 'Select'), ('multiselect', 'Multiselect')], max_length=11)),
                ('help_text', models.TextField(blank=True, help_text='Enter a help text to be displayed next to the input element')),
                ('options', jsonfield.fields.JSONField(blank=True, help_text='Enter valid JSON of the form [[key, label], [key, label], ...]', null=True)),
                ('required', models.BooleanField()),
            ],
            options={
                'ordering': ('key',),
                'verbose_name': 'DetailKey',
                'verbose_name_plural': 'DetailKeys',
            },
        ),
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('details', jsonfield.fields.JSONField(blank=True, null=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('user',),
                'verbose_name': 'Profile',
                'verbose_name_plural': 'Profiles',
            },
        ),
    ]



from rest_framework import filters, viewsets
from rest_framework.authentication import SessionAuthentication, TokenAuthentication
from rest_framework.pagination import PageNumberPagination

from django_filters.rest_framework import DjangoFilterBackend

from daiquiri.core.permissions import HasModelPermission
from daiquiri.stats.models import Record
from daiquiri.stats.serializers import RecordSerializer


class RecordViewSetPagination(PageNumberPagination):
    page_size = 1000
    page_size_query_param = 'page_size'
    max_page_size = 10000


class RecordViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)
    http_method_names = ['get', ]

    queryset = Record.objects.all()
    serializer_class = RecordSerializer
    pagination_class = RecordViewSetPagination

    filter_backends = (DjangoFilterBackend, filters.SearchFilter, filters.OrderingFilter)
    filterset_fields = ('user', 'resource_type', 'time')
    search_fields = ('user', 'resource_type', 'time')
    ordering_fields = ('time', 'resource_type')



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-13 15:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0002_meta'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='queryjob',
            managers=[
            ],
        ),
        migrations.AddField(
            model_name='queryjob',
            name='actual_query',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='queryjob',
            name='query_language',
            field=models.CharField(choices=[('adql', 'ADQL'), ('mysql', 'MySQL SQL')], default='mysql', max_length=8),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='queue',
            field=models.CharField(choices=[('default', 'Default')], max_length=16),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-05-16 11:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_auth', '0010_profile_consent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='profile',
            name='is_confirmed',
            field=models.BooleanField(default=False, help_text='Designates whether the user was confirmed by a manager.'),
        ),
        migrations.AlterField(
            model_name='profile',
            name='is_pending',
            field=models.BooleanField(default=False, help_text='Designates whether the user waiting on confirmation by a manager.'),
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _


class Record(models.Model):

    identifier = models.CharField(
        max_length=256,
        db_index=True,
        verbose_name=_('OAI identifier'),
    )
    datestamp = models.DateField(
        db_index=True,
        verbose_name=_('OAI datestamp'),
    )
    metadata_prefix = models.CharField(
        max_length=16, db_index=True,
        verbose_name=_('OAI metadataPrefix'),
    )
    set_spec = models.CharField(
        max_length=256, db_index=True, blank=True,
        verbose_name=_('OAI set spec'),
    )
    deleted = models.BooleanField(
        default=False, db_index=True,
        verbose_name=_('Deleted'),
    )
    resource_type = models.CharField(
        max_length=32, db_index=True,
        verbose_name=_('Resource type'),
    )
    resource_id = models.CharField(
        max_length=256, db_index=True, blank=True,
        verbose_name=_('Resource id'),
    )

    class Meta:
        db_table = 'records'

        ordering = ('-datestamp', )

        verbose_name = _('Record')
        verbose_name_plural = _('Records')

    def __str__(self):
        return self.identifier



from django.http import HttpResponse

from daiquiri.core.renderers.voresource import VoresourceRenderer
from daiquiri.core.renderers.vosi import AvailabilityRenderer, CapabilitiesRenderer

from .vo import get_availability, get_capabilities, get_resource


def resource(request):
    return HttpResponse(VoresourceRenderer().render(get_resource()), content_type="application/xml")


def availability(request):
    return HttpResponse(AvailabilityRenderer().render(get_availability()), content_type="application/xml")


def capabilities(request):
    return HttpResponse(CapabilitiesRenderer().render(get_capabilities()), content_type="application/xml")



from django.http import HttpResponse
from django.shortcuts import render

from daiquiri.core.renderers.voresource import VoresourceRenderer
from daiquiri.core.renderers.vosi import AvailabilityRenderer, CapabilitiesRenderer, TablesetRenderer
from daiquiri.query.models import Example

from .vo import get_availability, get_capabilities, get_resource, get_tableset


def resource(request):
    return HttpResponse(VoresourceRenderer().render(get_resource()), content_type="application/xml")


def availability(request):
    return HttpResponse(AvailabilityRenderer().render(get_availability()), content_type="application/xml")


def capabilities(request):
    return HttpResponse(CapabilitiesRenderer().render(get_capabilities()), content_type="application/xml")


def tables(request):
    return HttpResponse(TablesetRenderer().render(get_tableset()), content_type="application/xml")


def examples(request):
    return render(request, 'tap/examples.html', {
        'examples': Example.objects.filter_by_access_level(request.user)
    })



from re import IGNORECASE, compile
from re import escape as rescape

from django import template
from django.utils.safestring import mark_safe

register = template.Library()

@register.filter(name='highlight')
def highlight(text, search):
    rgx = compile(rescape(search), IGNORECASE)
    return mark_safe(
        rgx.sub(
            lambda m: f'<b>{m.group()}</b>',
            text
        )
    )

@register.filter(name='underline')
def underline(text, search):
    rgx = compile(rescape(search), IGNORECASE)
    return mark_safe(
        rgx.sub(
            lambda m: f'<u>{m.group()}</u>',
            text
        )
    )



# inspired by https://djangosnippets.org/snippets/10572/

from collections import OrderedDict

from django.contrib.auth.hashers import CryptPasswordHasher, mask_hash
from django.utils.crypto import constant_time_compare, get_random_string
from django.utils.encoding import force_str
from django.utils.translation import gettext_noop as _


class CrypdSHA512PasswordHasher(CryptPasswordHasher):

    algorithm = 'crypt_sha512'

    def salt(self):
        return '$6$' + get_random_string(8)

    def encode(self, password, salt):
        crypt = self._load_library()
        data = crypt.crypt(force_str(password), salt)
        return f"{self.algorithm}{data}"

    def verify(self, password, encoded):
        crypt = self._load_library()
        algorithm, rest = encoded.split('$', 1)
        salt, hash = rest.rsplit('$', 1)
        salt = '$' + salt
        assert algorithm == self.algorithm
        return constant_time_compare(f'{salt}${hash}', crypt.crypt(force_str(password), salt))

    def safe_summary(self, encoded):
        algorithm, prefix, salt, hash = encoded.split('$')
        assert algorithm == self.algorithm
        return OrderedDict([
            (_('algorithm'), algorithm),
            (_('prefix'), prefix),
            (_('salt'), mask_hash(salt)),
            (_('hash'), mask_hash(hash)),
        ])



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-30 14:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        ('daiquiri_metadata', '0010_meta'),
    ]

    operations = [
        migrations.CreateModel(
            name='Directory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(help_text='Path of the directory.', max_length=256, verbose_name='Path')),
                ('access_level', models.CharField(choices=[(b'PRIVATE', 'Private - access must be granted by group'), (b'INTERNAL', 'Internal - logged in users can access'), (b'PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Access level')),
                ('metadata_access_level', models.CharField(choices=[(b'PRIVATE', 'Private - access must be granted by group'), (b'INTERNAL', 'Internal - logged in users can access'), (b'PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Metadata access level')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups which have access to this function.', to='auth.Group', verbose_name='Groups')),
            ],
            options={
                'ordering': ('path',),
                'verbose_name': 'Directory',
                'verbose_name_plural': 'Directory',
                'permissions': (('view_function', 'Can view Directory'),),
            },
        ),
    ]



import pytest

from django.urls import reverse

users = (
    ('admin', 'admin'),
    ('manager', 'manager'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'list': {
        'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 403
    },
    'create': {
        'admin': 405, 'manager': 405, 'user': 405, 'anonymous': 403
    }
}

urlnames = {
    'list': 'contact:status-list'
}


@pytest.mark.parametrize(('username', 'password'), users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == status_map['list'][username], response.json()

    if response.status_code == 200:
        assert len(response.json()) == 3


@pytest.mark.parametrize(('username', 'password'), users)
def test_create(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url, {})
    assert response.status_code == status_map['create'][username], response.json()



from django.conf import settings

from rest_framework import serializers

from ..models import Column, Function, Schema, Table


class ColumnSerializer(serializers.ModelSerializer):

    if settings.METADATA_COLUMN_PERMISSIONS:
        groups = serializers.SerializerMethodField()

    class Meta:
        model = Column

        # only show access_level, metadata_access_level, and groups when
        # settings.METADATA_COLUMN_PERMISSIONS is set
        if settings.METADATA_COLUMN_PERMISSIONS:
            fields = (
                'order',
                'name',
                'description',
                'unit',
                'ucd',
                'utype',
                'principal',
                'std',
                'access_level',
                'metadata_access_level',
                'groups'
            )
        else:
            fields = (
                'order',
                'name',
                'description',
                'unit',
                'ucd',
                'utype',
                'principal',
                'std'
            )

    def get_groups(self, obj):
        return [group.name for group in obj.groups.all()]


class TableSerializer(serializers.ModelSerializer):

    groups = serializers.SerializerMethodField()
    columns = ColumnSerializer(many=True, read_only=True)

    class Meta:
        model = Table
        fields = (
            'order',
            'name',
            'title',
            'description',
            'long_description',
            'attribution',
            'license',
            'doi',
            'type',
            'utype',
            'access_level',
            'metadata_access_level',
            'groups',
            'columns'
        )

    def get_groups(self, obj):
        return [group.name for group in obj.groups.all()]


class SchemaSerializer(serializers.ModelSerializer):

    groups = serializers.SerializerMethodField()
    tables = TableSerializer(many=True, read_only=True)

    class Meta:
        model = Schema
        fields = (
            'order',
            'name',
            'title',
            'description',
            'long_description',
            'attribution',
            'license',
            'doi',
            'utype',
            'access_level',
            'metadata_access_level',
            'groups',
            'tables'
        )

    def get_groups(self, obj):
        return [group.name for group in obj.groups.all()]


class FunctionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Function
        fields = (
            'order',
            'name',
            'query_string'
        )



from rest_framework import serializers

from daiquiri.core.serializers import JSONDictField, JSONListField


class DublincoreSerializer(serializers.Serializer):

    identifier = serializers.ReadOnlyField()
    title = serializers.ReadOnlyField()
    description = serializers.SerializerMethodField()
    publisher = serializers.SerializerMethodField()
    subjects = serializers.ReadOnlyField()

    def get_description(self, obj):
        return obj['content']['description']

    def get_publisher(self, obj):
        return obj['curation']['publisher']


class VoresourceSerializer(serializers.Serializer):

    identifier = serializers.ReadOnlyField()
    title = serializers.ReadOnlyField()
    created = serializers.ReadOnlyField()
    updated = serializers.ReadOnlyField()
    type = serializers.ReadOnlyField()
    status = serializers.ReadOnlyField()

    curation = JSONDictField(default={})
    content = JSONDictField(default={})
    capabilities = JSONListField(default=[])
    tableset = JSONListField(default=[])

    full = serializers.ReadOnlyField(default=None)
    managed_authority = serializers.ReadOnlyField(default=None)
    managing_org = serializers.ReadOnlyField(default=None)



from django.conf import settings

from rest_framework import serializers


class DublincoreSerializer(serializers.Serializer):

    title = serializers.CharField()
    creators = serializers.ReadOnlyField(default={'name': settings.SITE_CREATOR})
    subjects = serializers.ReadOnlyField(default=settings.SITE_SUBJECTS)
    publisher = serializers.ReadOnlyField(default=settings.SITE_PUBLISHER)
    date = serializers.CharField(default=settings.SITE_CREATED)
    identifier = serializers.CharField(source='doi')
    rights = serializers.CharField(default=settings.LICENSE_URLS.get('CC0'))


class DataciteSerializer(serializers.Serializer):

    identifier = serializers.CharField(source='doi')
    creators = serializers.ReadOnlyField(default=[
        {
            'name': settings.SITE_CREATOR
        }
    ])
    title = serializers.CharField()
    publisher = serializers.ReadOnlyField(default=settings.SITE_PUBLISHER)
    publication_year = serializers.CharField(default=settings.SITE_CREATED.split('-')[0])
    subjects = serializers.ReadOnlyField(default=settings.SITE_SUBJECTS)
    updated = serializers.CharField(default=settings.SITE_UPDATED)
    language = serializers.ReadOnlyField(default=settings.SITE_LANGUAGE)
    formats = serializers.ListField()
    license = serializers.CharField(default='CC0')
    license_url = serializers.CharField(default=settings.LICENSE_URLS.get('CC0'))
    resource_type = serializers.CharField(default='Dataset')
    alternate_identifiers = serializers.ReadOnlyField(default=[])
    related_identifiers = serializers.ReadOnlyField(default=[])



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-14 13:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0007_alter_validators_add_error_messages'),
        ('daiquiri_query', '0005_meta'),
    ]

    operations = [
        migrations.CreateModel(
            name='Example',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(blank=True, null=True)),
                ('name', models.CharField(max_length=256)),
                ('description', models.TextField(blank=True, null=True)),
                ('query_string', models.TextField()),
                ('groups', models.ManyToManyField(blank=True, to='auth.Group')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'Example',
                'verbose_name_plural': 'Examples',
                'permissions': (('view_example', 'Can view Example'),),
            },
        ),
    ]



from django.urls import include, path

from rest_framework import routers

from .views import MessagesView, contact
from .viewsets import ContactMessageViewSet, StatusViewSet

app_name = 'contact'

router = routers.DefaultRouter()
router.register(r'messages', ContactMessageViewSet, basename='message')
router.register(r'status', StatusViewSet, basename='status')

urlpatterns = [
    path('', contact, name='contact'),
    path('messages/', MessagesView.as_view(), name='messages'),

    # rest api
    path('api/', include(router.urls)),
]



# Generated by Django 2.1.4 on 2019-07-08 12:00

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0023_queryjob_uploads'),
    ]

    def migrate_data(apps, schema_editor):

        QueryJob = apps.get_model('daiquiri_query', 'queryjob')

        for job in QueryJob.objects.all():
            if job.metadata in ['', '""']:
                job.metadata = None
            job.save()

    operations = [
        migrations.AlterField(
            model_name='queryjob',
            name='metadata',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
        migrations.RunPython(migrate_data),
    ]



from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from .models import Job


class JobAdmin(admin.ModelAdmin):
    search_fields = ['id', 'job_type', 'owner__username', 'phase']
    list_display = ['id', 'job_type', 'owner', 'run_id', 'phase', 'creation_time']

    list_filter = ['job_type', 'phase']
    ordering = ['-creation_time']

    def abort_job(self, request, queryset):
        for job in queryset:
            try:
                job.abort()
            except NotImplementedError:
                pass

    def archive_job(self, request, queryset):
        for job in queryset:
            try:
                job.archive()
            except NotImplementedError:
                pass

    abort_job.short_description = _('Abort selected jobs')
    archive_job.short_description = _('Archive selected jobs')


admin.site.register(Job, JobAdmin)



from django.utils.translation import gettext_lazy as _

import daiquiri.core.env as env

METADATA_COLUMN_PERMISSIONS = False
METADATA_BASE_URL = None

# This setting sets the default width of the columns in the results of
# query and in the 'serve' app. If the column is not found here, then
# default is used. The key for the specific column is of the form
# 'schema_name.table_name.column_name'
METADATA_COLUMN_WIDTH = {
    'default': 200,
}

ARCHIVE_BASE_PATH = env.get_abspath('ARCHIVE_BASE_PATH')

LICENSE_NONE = ''
LICENSE_CC0 = 'CC0'
LICENSE_PD = 'PD'
LICENSE_BY = 'BY'
LICENSE_BY_SA = 'BY_SA'
LICENSE_BY_ND = 'BY_ND'
LICENSE_BY_NC = 'BY_NC'
LICENSE_BY_NC_SA = 'BY_NC_SA'
LICENSE_BY_NC_ND = 'BY_NC_ND'

LICENSE_CHOICES = (
    (LICENSE_NONE, _('---')),
    (LICENSE_CC0, _('CC0 1.0 Universal (CC0 1.0)')),
    (LICENSE_PD, _('Public Domain Mark')),
    (LICENSE_BY, _('Attribution 4.0 International (CC BY 4.0)')),
    (LICENSE_BY_SA, _('Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)')),
    (LICENSE_BY_ND, _('Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)')),
    (LICENSE_BY_NC, _('Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)')),
    (LICENSE_BY_NC_SA, _('Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)')),
    (LICENSE_BY_NC_ND, _('Attribution-NonCommercial-NoDerivatives 4.0 International (CC BY-NC-ND 4.0)'))
)

LICENSE_URLS = {
    LICENSE_CC0: 'https://creativecommons.org/publicdomain/zero/1.0/',
    LICENSE_PD: None,
    LICENSE_BY: 'https://creativecommons.org/licenses/by/4.0/',
    LICENSE_BY_SA: 'https://creativecommons.org/licenses/by-sa/4.0/',
    LICENSE_BY_ND: 'https://creativecommons.org/licenses/by-nd/4.0/',
    LICENSE_BY_NC: 'https://creativecommons.org/licenses/by-nc/4.0/',
    LICENSE_BY_NC_SA: 'https://creativecommons.org/licenses/by-nc-sa/4.0/',
    LICENSE_BY_NC_ND: 'https://creativecommons.org/licenses/by-nc-nd/4.0/'
}

LICENSE_IDENTIFIERS = {
    LICENSE_CC0: 'CC0-1.0',
    LICENSE_PD: None,
    LICENSE_BY: 'CC-BY-4.0',
    LICENSE_BY_SA: 'CC-BY-SA-4.0',
    LICENSE_BY_ND: 'CC-BY-ND-4.0',
    LICENSE_BY_NC: 'CC-BY-NC-4.0',
    LICENSE_BY_NC_SA: 'CC-BY-NC-SA-4.0',
    LICENSE_BY_NC_ND: 'CC-BY-NC-ND-4.0'
}



# Generated by Django 3.2.12 on 2022-02-04 14:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0032_data_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='column',
            name='access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Access level'),
        ),
        migrations.AlterField(
            model_name='column',
            name='datatype',
            field=models.CharField(blank=True, default='', help_text='The datatype of the column on the database server.', max_length=256, verbose_name='Datatype'),
        ),
        migrations.AlterField(
            model_name='column',
            name='description',
            field=models.TextField(blank=True, default='', help_text='A brief description of the column to be displayed in the user interface.', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='column',
            name='metadata_access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Metadata access level'),
        ),
        migrations.AlterField(
            model_name='column',
            name='ucd',
            field=models.CharField(blank=True, default='', max_length=256, verbose_name='IVOA UCDs'),
        ),
        migrations.AlterField(
            model_name='column',
            name='unit',
            field=models.CharField(blank=True, default='', max_length=256, verbose_name='Unit'),
        ),
        migrations.AlterField(
            model_name='column',
            name='utype',
            field=models.CharField(blank=True, default='', max_length=256, verbose_name='IVOA Utype'),
        ),
        migrations.AlterField(
            model_name='function',
            name='access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Access level'),
        ),
        migrations.AlterField(
            model_name='function',
            name='description',
            field=models.TextField(blank=True, default='', help_text='A brief description of the function to be displayed in the user interface.', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='function',
            name='metadata_access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Metadata access level'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Access level'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='attribution',
            field=models.TextField(blank=True, default='', help_text='The desired attribution for the schema.', verbose_name='Attribution'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='contributors',
            field=models.JSONField(blank=True, default=list, verbose_name='Contributors'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='creators',
            field=models.JSONField(blank=True, default=list, verbose_name='Creators'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='description',
            field=models.TextField(blank=True, default='', help_text='A brief description of the schema to be displayed in the user interface.', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='doi',
            field=models.CharField(blank=True, default='', max_length=256, verbose_name='Digital object identifier'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='license',
            field=models.CharField(blank=True, default='', max_length=8, verbose_name='License'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='long_description',
            field=models.TextField(blank=True, default='', help_text='A more extensive description of the schema to be displayed on the public schema page.', verbose_name='Long description'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='metadata_access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Metadata access level'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='related_identifiers',
            field=models.JSONField(blank=True, default=list, verbose_name='Related Identifiers'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='title',
            field=models.CharField(blank=True, default='', help_text='Human readable title of the schema.', max_length=256, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='utype',
            field=models.CharField(blank=True, default='', max_length=256, verbose_name='IVOA Utype'),
        ),
        migrations.AlterField(
            model_name='table',
            name='access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Access level'),
        ),
        migrations.AlterField(
            model_name='table',
            name='attribution',
            field=models.TextField(blank=True, default='', help_text='The desired attribution for the table.', verbose_name='Attribution'),
        ),
        migrations.AlterField(
            model_name='table',
            name='contributors',
            field=models.JSONField(blank=True, default=list, verbose_name='Contributors'),
        ),
        migrations.AlterField(
            model_name='table',
            name='creators',
            field=models.JSONField(blank=True, default=list, verbose_name='Creators'),
        ),
        migrations.AlterField(
            model_name='table',
            name='description',
            field=models.TextField(blank=True, default='', help_text='A brief description of the table to be displayed in the user interface.', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='table',
            name='doi',
            field=models.CharField(blank=True, default='', max_length=256, verbose_name='Digital object identifier'),
        ),
        migrations.AlterField(
            model_name='table',
            name='license',
            field=models.CharField(blank=True, default='', max_length=8, verbose_name='License'),
        ),
        migrations.AlterField(
            model_name='table',
            name='long_description',
            field=models.TextField(blank=True, default='', help_text='A more extensive description of the table to be displayed on the public database page.', verbose_name='Long description'),
        ),
        migrations.AlterField(
            model_name='table',
            name='metadata_access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], default='PRIVATE', max_length=8, verbose_name='Metadata access level'),
        ),
        migrations.AlterField(
            model_name='table',
            name='related_identifiers',
            field=models.JSONField(blank=True, default=list, verbose_name='Related Identifiers'),
        ),
        migrations.AlterField(
            model_name='table',
            name='title',
            field=models.CharField(blank=True, default='', help_text='Human readable title of the table.', max_length=256, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='table',
            name='type',
            field=models.CharField(choices=[('table', 'table'), ('view', 'view')], default='table', max_length=8, verbose_name='Type of table'),
        ),
        migrations.AlterField(
            model_name='table',
            name='utype',
            field=models.CharField(blank=True, default='', max_length=256, verbose_name='IVOA Utype'),
        ),
    ]



# Generated by Django 4.2.20 on 2025-03-18 16:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0027_rename_job_to_query_job'),
    ]

    operations = [
        migrations.AlterField(
            model_name='queryarchivejob',
            name='files',
            field=models.JSONField(default=list, help_text='List of files in the archive.', verbose_name='Files'),
        ),
    ]



import sys

from django.core.management.base import BaseCommand

from daiquiri.core.adapter import DownloadAdapter
from daiquiri.metadata.models import Table


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('schema', help='the schema for the table to dump')
        parser.add_argument('table', help='the table to dump')
        parser.add_argument('format', help='the format for the dump')

    def handle(self, *args, **options):
        table = Table.objects.filter(schema__name=options['schema']).get(name=options['table'])

        for line in DownloadAdapter().generate(options['format'], options['schema'], options['table'],
                                               table.columns.values()):
            sys.stdout.write(line)



from django.contrib.auth.models import User
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def handle(self, *args, **options):
        User.objects.create_superuser('admin', 'admin@example.com', 'admin')



import csv
import re
import sys

from django.contrib.auth.models import User
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            "id_list_file",
            type=str,
            help="required list of user ids to delete in plain text format, "
            + "user ids have to be at the beginning of the line, "
            + "supports commenting lines out: if a line does "
            + "not start with an integer it will be skipped",
        )
        parser.add_argument(
            "-n",
            "--dry_run",
            action="store_true",
            default=False,
            help="dry run, does not delete anything but print what "
            + "would have happened",
        )

    def make_user_id_list(self, filename):
        user_ids = []
        try:
            filecontent = open(filename, encoding="utf-8", newline="")
        except Exception as e:
            print("Error reading id list file. " + str(e))
            sys.exit(1)
        else:
            csv_reader = csv.DictReader(filecontent, dialect="unix")
            for dic in csv_reader:
                m = re.search(r"^[0-9]+$", dic["id"])
                if bool(m) is True:
                    user_ids.append(dic)
        return sorted(user_ids, key=lambda k: k["id"])

    def delete_users(self, user_dic, dry_run):
        for dic in user_dic:
            try:
                u = User.objects.get(id=dic["id"])
                if dry_run is False:
                    print("Delete user {}, {}".format(dic["id"], dic["email"]))
                    u.delete()
                else:
                    print("Would have deleted user: {}, {}".format(dic["id"], dic["email"]))
            except Exception as e:
                print("Error deleting user " + str(id) + ". " + str(e))

    def handle(self, *args, **options):
        user_ids = self.make_user_id_list(options["id_list_file"])
        if options["dry_run"] is True:
            self.delete_users(user_ids, options["dry_run"])
        else:
            print("\nYou are about to delete " + str(len(user_ids)) + " users.")
            val = input("Are you sure? If so please enter 'yes' to continue?    ")
            if val == "yes":
                self.delete_users(user_ids, options["dry_run"])
            else:
                print("\nAborted.\n")



import os

from django import template
from django.template.defaultfilters import stringfilter
from django.urls import reverse
from django.utils.encoding import force_str
from django.utils.safestring import mark_safe

from markdown import markdown as markdown_function

from ..utils import get_script_alias

register = template.Library()


@register.simple_tag(takes_context=True)
def base_url(context):
    return get_script_alias(context.request) + '/'


@register.simple_tag(takes_context=True)
def absolute_url(context, name, *args):
    return context.request.build_absolute_uri(reverse(name, args=args))


@register.filter(name='next')
def next(value, arg):
    try:
        return value[int(arg)+1]
    except (ValueError, IndexError):
        return None


@register.filter(is_safe=True)
@stringfilter
def markdown(value):
    return mark_safe(markdown_function(force_str(value)))


@register.filter(is_safe=True)
@stringfilter
def semicolonbr(value):
    return mark_safe(value.replace(';', '<br />'))


@register.filter(is_safe=True)
@stringfilter
def to_safe_str(value):
    return mark_safe(str(value))


@register.filter()
def default_blank(value):
    return value if value else ''


@register.filter()
def default_blank_unit(value, unit):
    return str(value) + ' ' + unit if value else ''


@register.filter()
def basename(value):
    return os.path.basename(value)


@register.filter()
def startswith(text, starts):
    if isinstance(text, str):
        return text.startswith(starts)
    return False



import json
from collections import OrderedDict

from django.conf import settings
from django.core.cache import cache
from django.utils.translation import gettext as _

from rest_framework.exceptions import ValidationError

from queryparser.adql import ADQLQueryTranslator
from queryparser.exceptions import QueryError, QuerySyntaxError

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.utils import filter_by_access_level
from daiquiri.metadata.models import Column, Function, Schema, Table

from .utils import get_default_table_name, get_indexed_objects, get_max_active_jobs, get_quota, get_user_schema_name


def check_quota(job):
    # get the model from the instance to prevent circular inclusion
    QueryJob = type(job)

    if QueryJob.objects.get_size(job.owner) > get_quota(job.owner):
        raise ValidationError({
            'query': [_('Quota is exceeded. Please remove some of your jobs.')]
        })


def check_number_of_active_jobs(job):
    # get the model from the instance to prevent circular inclusion
    QueryJob = type(job)

    max_active_jobs = get_max_active_jobs(job.owner)
    if max_active_jobs and max_active_jobs <= QueryJob.objects.get_active(job.owner).count():
        raise ValidationError({
            'query': [_('Too many active jobs. Please abort some of your active jobs or '
                        'wait until they are completed.')]
        })


def process_schema_name(user, schema_name):
    user_schema_name = get_user_schema_name(user)

    if schema_name:
        if schema_name == user_schema_name:
            return schema_name
        else:
            raise ValidationError({
                'schema_name': [_('Only the user schema is allowed.')]
            })
    else:
        return user_schema_name


def process_table_name(table_name):
    if table_name:
        # the tablename was checked by the TableNameValidator
        return table_name
    else:
        return get_default_table_name()


def process_query_language(user, query_language):
    # get the possible query languages for this user and create a map
    query_language_map = {}
    for item in filter_by_access_level(user, settings.QUERY_LANGUAGES):
        query_language_map['{key}-{version}'.format(**item)] = '{key}-{version}'.format(**item)
        query_language_map['{key}'.format(**item)] = '{key}-{version}'.format(**item)

    # check if a query language is set
    if query_language:
        query_language = query_language.lower()

        if query_language in query_language_map:
            return query_language_map[query_language]
        else:
            raise ValidationError({
                'query_language': [_('This query language is not supported.')]
            })

    else:
        # return the default query_language
        return settings.QUERY_LANGUAGES[0]['key']


def process_queue(user, queue):
    # get the possible queues for this user
    queues = filter_by_access_level(user, settings.QUERY_QUEUES)

    # check if a queue is set
    if queue:
        queue = queue.lower()

        # check if this queue is in the possible queues
        try:
            next(item for item in queues if item['key'] == queue)
        except StopIteration as e:
            raise ValidationError({
                'queue': [_('This queue is not supported.')]
            }) from e

        return queue
    else:
        # set the default queue
        return queues[0]['key']


def process_response_format(response_format):
    if response_format:
        response_format = response_format.lower()

        if response_format in [item['key'] for item in settings.QUERY_DOWNLOAD_FORMATS]:
            return response_format
        else:
            raise ValidationError({
                'response_format': [_('This response format is not supported.')]
            })
    else:
        # return the default response_format
        return settings.QUERY_DEFAULT_DOWNLOAD_FORMAT


def translate_query(query_language, query):
    # get the adapter
    adapter = DatabaseAdapter()

    # translate adql -> mysql string
    if query_language == 'adql-2.0':
        try:
            translator = cache.get_or_set('translator', ADQLQueryTranslator(), 3600)
            translator.set_query(query)

            if adapter.database_config['ENGINE'] == 'django.db.backends.mysql':
                return translator.to_mysql()
            elif adapter.database_config['ENGINE'] == 'django.db.backends.postgresql':
                return translator.to_postgresql()
            else:
                raise Exception('Unknown database engine')

        except QuerySyntaxError as e:
            raise ValidationError({
                'query': {
                    'messages': [_('There has been an error while translating your query.')],
                    'positions': json.dumps(e.syntax_errors),
                }
            }) from e

        except QueryError as e:
            raise ValidationError({
                'query': {
                    'messages': e.messages,
                }
            }) from e

    else:
        return query


def process_query(query):
    # get the adapter
    adapter = DatabaseAdapter()

    try:
        if adapter.database_config['ENGINE'] == 'django.db.backends.mysql':

            from queryparser.mysql import MySQLQueryProcessor
            processor = MySQLQueryProcessor(query)

        elif adapter.database_config['ENGINE'] == 'django.db.backends.postgresql':

            from queryparser.postgresql import PostgreSQLQueryProcessor

            if settings.QUERY_PROCESSOR_CACHE:
                processor = cache.get_or_set('processor', PostgreSQLQueryProcessor(), 3600)
            else:
                processor = PostgreSQLQueryProcessor()

            # first run to replace with get_indexed_objects
            processor.set_query(query)
            processor.process_query(indexed_objects=get_indexed_objects(), replace_schema_name={
                'TAP_SCHEMA': settings.TAP_SCHEMA,
                'tap_schema': settings.TAP_SCHEMA,
                'TAP_UPLOAD': settings.TAP_UPLOAD,
                'tap_upload': settings.TAP_UPLOAD,
            })

            # second run
            processor.set_query(processor.query)
            processor.process_query()

        else:
            raise Exception('Unknown database engine')

    except QuerySyntaxError as e:
        raise ValidationError({
            'query': {
                'messages': [_('There has been an error while parsing your query.')],
                'positions': json.dumps(e.syntax_errors),
            }
        }) from e

    except QueryError as e:
        raise ValidationError({
            'query': {
                'messages': [e.messages, ],
            }
        }) from e

    return processor


def process_display_columns(processor_display_columns):
    # process display_columns to expand *
    display_columns = []
    for processor_display_column, original_column in processor_display_columns:
        if processor_display_column == '*':
            schema_name, table_name, tmp = original_column
            columns = Column.objects.filter(table__schema__name=schema_name) \
                                    .filter(table__name=table_name) \
                                    .order_by('order')
            for column in columns:
                display_columns.append((column.name, (schema_name, table_name, column.name)))

        else:
            display_columns.append((processor_display_column, original_column))

    # check for duplicate columns in display_columns
    seen = set()
    errors = []
    for display_column_name, display_column in display_columns:
        if display_column_name not in seen:
            seen.add(display_column_name)
        else:
            errors.append(_('Duplicate column name %(column)s') % {
                'column': display_column_name
            })

    if errors:
        raise ValidationError({
            'query': list(errors)
        })

    return OrderedDict(display_columns)

def process_user_columns(job, processor_tables):
    '''Process the columns of user tables
    '''
    columns = []

    # get type from input job itself.
    QueryJob = type(job)

    for schema_name, table_name in processor_tables:
        # check if a user table is part of the table list
        if schema_name == get_user_schema_name(job.owner):
            user_job = QueryJob.objects.filter(owner=job.owner) \
                                       .exclude(phase=QueryJob.PHASE_ARCHIVED) \
                                       .get(table_name=table_name)
            columns.extend(user_job.metadata['columns'])

    return columns

def check_permissions(user, keywords, tables, columns, functions):
    messages = []

    # removed, not sure what this was
    # check keywords against whitelist
    # for keywords in keywords:
    #     pass

    # loop over tables to check permissions on schemas/tables
    for schema_name, table_name in tables:

        # check permission on schema
        if schema_name is None:
            # schema_name must not be null, move to next table
            messages.append(_('No schema given for table %s.') % table_name)
            continue
        elif schema_name == get_user_schema_name(user):
            # all tables are allowed move to next table
            continue
        elif schema_name == settings.TAP_UPLOAD:
            # all tables are allowed move to next table
            continue
        else:
            # check permissions on the schema
            try:
                schema = Schema.objects.filter_by_access_level(user).get(name=schema_name)
            except Schema.DoesNotExist:
                # schema not found or not allowed, move to next table
                messages.append(_('Schema %s not found.') % schema_name)
                continue

        # check permission on table
        if table_name is None:
            # table_name must not be null, move to next table
            messages.append(_('No table given for schema %s.') % schema_name)
            continue
        else:
            try:
                Table.objects.filter_by_access_level(user).filter(schema=schema).get(name=table_name)
            except Table.DoesNotExist:
                # table not found or not allowed, move to next table
                messages.append(_('Table %s not found.') % table_name)
                continue

    # loop over columns to check permissions or just to see if they are there,
    # but only if no error messages where appended so far
    if not messages:

        for schema_name, table_name, column_name in columns:

            if schema_name in [None, get_user_schema_name(user), settings.TAP_UPLOAD] \
                    or table_name is None \
                    or column_name is None:
                # doesn't need to be checked, move to next column
                continue
            else:
                if not settings.METADATA_COLUMN_PERMISSIONS:
                    # just check if the column exist
                    if column_name == '*':
                        # doesn't need to be checked, move to next table
                        continue

                    else:
                        try:
                            Column.objects.filter(table__schema__name=schema_name).filter(table__name=table_name).get(name=column_name)
                        except Column.DoesNotExist:
                            messages.append(_('Column %s not found.') % column_name)
                            continue
                else:
                    try:
                        schema = Schema.objects.filter_by_access_level(user).get(name=schema_name)
                    except Schema.DoesNotExist:
                        messages.append(_('Schema %s not found.') % schema_name)
                        continue

                    try:
                        table = Table.objects.filter_by_access_level(user).filter(schema=schema).get(name=table_name)
                    except Table.DoesNotExist:
                        messages.append(_('Table %s not found.') % table_name)
                        continue

                    if column_name == '*':
                        columns = Column.objects.filter_by_access_level(user).filter(table=table)
                        actual_columns = DatabaseAdapter().fetch_columns(schema_name, table_name)

                        column_names_set = {column.name for column in columns}
                        actual_column_names_set = {column['name'] for column in actual_columns}

                        if column_names_set != actual_column_names_set:
                            messages.append(_('The asterisk (*) is not allowed for this table.'))
                            continue

                    else:
                        try:
                            Column.objects.filter_by_access_level(user).filter(table=table).get(name=column_name)
                        except Column.DoesNotExist:
                            messages.append(_('Column %s not found.') % column_name)
                            continue

    # check permissions on functions
    for function_name in functions:

        # check permission on function
        queryset = Function.objects.filter(name=function_name)

        # forbid the function if it is in metadata.functions, and the user doesn't have access.
        if queryset and not queryset.filter_by_access_level(user):
            messages.append(_('Function %s is not allowed.') % function_name)
        else:
            continue

    # return the error stack
    return list(set(messages))



from django.http import HttpResponse
from django.shortcuts import render

from daiquiri.core.renderers.vosi import AvailabilityRenderer, CapabilitiesRenderer

from .adapter import DatalinkAdapter
from .settings import DATALINK_CUSTOM_SEMANTICS
from .vo import get_availability, get_capabilities


def datalink(request, ID):
    adapter = DatalinkAdapter()
    context = adapter.get_context_data(request, ID=ID)
    context['custom_semantics'] = DATALINK_CUSTOM_SEMANTICS
    return render(request, 'datalink/datalink.html', context)

def datalink_semantics(request):
    context = { 'custom_semantics': DATALINK_CUSTOM_SEMANTICS }
    return render(request, 'datalink/datalink-semantics.html', context)

def availability(request):
    return HttpResponse(AvailabilityRenderer().render(get_availability()), content_type="application/xml")


def capabilities(request):
    return HttpResponse(CapabilitiesRenderer().render(get_capabilities()), content_type="application/xml")



class DublincoreRendererMixin:

    def render_dublincore(self, metadata):
        self.start('oai_dc:dc', {
            'xmlns:oai_dc': 'http://www.openarchives.org/OAI/2.0/oai_dc/',
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xsi:schemaLocation': 'http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd'
        })

        self.node('dc:identifier', {}, metadata.get('identifier'))
        self.node('dc:title', {}, metadata.get('title'))

        creators = metadata.get('creators')
        if isinstance(creators, list):
            for creator in creators:
                self.node('dc:creator', {}, creator.get('name'))

        contributors = metadata.get('contributors')
        if isinstance(contributors, list):
            for contributor in contributors:
                self.node('dc:contributor', {}, contributor.get('name'))

        for subject in metadata.get('subjects', []):
            self.node('dc:subject', {}, subject.get('subject'))

        self.node('dc:publisher', {}, metadata.get('publisher'))

        if metadata.get('description'):
            self.node('dc:description', {}, metadata.get('description'))
        if metadata.get('date'):
            self.node('dc:date', {}, metadata.get('date'))
        if metadata.get('rights'):
            self.node('dc:rights', {}, metadata.get('rights'))
        if metadata.get('type'):
            self.node('dc:type', {}, metadata.get('type'))

        self.end('oai_dc:dc')



from django.utils.translation import gettext_lazy as _

from rest_framework import mixins, viewsets
from rest_framework.exceptions import ParseError

from .serializers import ChoicesSerializer


class ChoicesViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    serializer_class = ChoicesSerializer


class RowViewSetMixin:

    def _get_query_params(self, columns):
        # get the ordering
        ordering = self.request.GET.get('ordering')

        # get the search string
        search = self.request.GET.get('search')

        # get the page from the querystring and make sure it is an int
        try:
            page = int(self.request.GET.get('page', '1'))
        except ValueError as e:
            raise ParseError(_('page must be an integer')) from e

        # get the page_size from the querystring and make sure it is an int
        try:
            page_size = int(self.request.GET.get('page_size', '30'))
        except ValueError as e:
            raise ParseError(_('page_size must be an integer')) from e

        # get additional filters from the querystring
        filters = {}
        try:
            column_names = [column.name for column in columns]
        except AttributeError:
            column_names = [column['name'] for column in columns]

        filters = {}
        for key, value in self.request.GET.items():
            if key in column_names:
                filters[key] = value

        return ordering, page, page_size, search, filters

    def _get_next_url(self, page, page_size, count):
        if page * page_size < count:
            querydict = self.request.GET.copy()
            querydict['page'] = page + 1
            return self.request.build_absolute_uri(self.request.path_info + '?' + querydict.urlencode())
        else:
            return None

    def _get_previous_url(self, page):
        if page > 1:
            querydict = self.request.GET.copy()
            querydict['page'] = page - 1
            return self.request.build_absolute_uri(self.request.path_info + '?' + querydict.urlencode())
        else:
            return None



import logging

from django.conf import settings

from daiquiri.core.utils import import_class

from .adapter import OaiAdapter
from .models import Record

logger = logging.getLogger(__name__)


def get_metadata_format(metadata_prefix):
    return next(metadata_format for metadata_format in settings.OAI_METADATA_FORMATS
                if metadata_format['prefix'] == metadata_prefix)


def get_renderer(metadata_prefix):
    renderer_class = get_metadata_format(metadata_prefix)['renderer_class']
    return import_class(renderer_class)()


def update_records(resource_type, resource):
    logger.debug('update_records %s %s', resource_type, resource)

    adapter = OaiAdapter()

    if resource_type in adapter.resource_types:
        try:
            resource_id, identifier, datestamp, set_spec, public = adapter.get_record(resource_type, resource)
        except TypeError as e:
            raise RuntimeError(f'Could not obtain record for {resource_type} {resource}') from e

        if public is True:
            for metadata_prefix in adapter.get_metadata_prefixes(resource_type):
                try:
                    record = Record.objects.get(identifier=identifier, metadata_prefix=metadata_prefix)
                except Record.DoesNotExist:
                    record = Record(identifier=identifier, metadata_prefix=metadata_prefix)

                record.datestamp = datestamp
                record.set_spec = set_spec
                record.deleted = False
                record.resource_type = resource_type
                record.resource_id = resource_id
                record.save()
        else:
            delete_records(resource_type, resource)


def delete_records(resource_type, resource):
    logger.debug('delete_records %s %s', resource_type, resource)

    adapter = OaiAdapter()

    if resource_type in adapter.resource_types:
        try:
            resource_id, identifier, datestamp, set_spec, public = adapter.get_record(resource_type, resource)
        except TypeError as e:
            raise RuntimeError(f'Could not obtain record for {resource_type} {resource}') from e

        for metadata_prefix in adapter.get_metadata_prefixes(resource_type):
            try:
                record = Record.objects.get(identifier=identifier, metadata_prefix=metadata_prefix)
                record.datestamp = datestamp
                record.set_spec = set_spec
                record.deleted = True
                record.save()
            except Record.DoesNotExist:
                pass



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-03-30 15:37
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Job',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('run_id', models.CharField(blank=True, max_length=256)),
                ('phase', models.PositiveSmallIntegerField(choices=[(1, 'PENDING'), (2, 'QUEUED'), (3, 'EXECUTING'), (4, 'COMPLETED'), (5, 'ERROR'), (6, 'ABORTED'), (7, 'UNKNOWN'), (8, 'HELD'), (9, 'SUSPENDED'), (10, 'ARCHIVED')])),
                ('start_time', models.DateTimeField(blank=True, null=True)),
                ('end_time', models.DateTimeField(blank=True, null=True)),
                ('execution_duration', models.PositiveIntegerField(blank=True, null=True)),
                ('destruction_time', models.DateTimeField(blank=True, null=True)),
                ('job_type', models.PositiveSmallIntegerField(choices=[(1, 'Query')])),
                ('owner', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('start_time',),
                'verbose_name': 'Job',
                'verbose_name_plural': 'Jobs',
                'permissions': (('view_job', 'Can view Job'),),
            },
        ),
    ]



from django.db import models

from .constants import ACCESS_LEVEL_INTERNAL, ACCESS_LEVEL_PUBLIC


class AccessLevelQuerySet(models.QuerySet):

    def filter_by_access_level(self, user):
        if not user or user.is_anonymous:
            return self.filter(access_level=ACCESS_LEVEL_PUBLIC)
        else:
            q = models.Q(access_level=ACCESS_LEVEL_PUBLIC) | \
                models.Q(access_level=ACCESS_LEVEL_INTERNAL) | \
                models.Q(groups__in=user.groups.all())
            return self.filter(q).distinct()

    def filter_by_metadata_access_level(self, user):
        if not user or user.is_anonymous:
            return self.filter(metadata_access_level=ACCESS_LEVEL_PUBLIC)
        else:
            q = models.Q(metadata_access_level=ACCESS_LEVEL_PUBLIC) | \
                models.Q(metadata_access_level=ACCESS_LEVEL_INTERNAL) | \
                models.Q(groups__in=user.groups.all())
            return self.filter(q).distinct()


class AccessLevelManager(models.Manager):

    def get_queryset(self):
        return AccessLevelQuerySet(self.model, using=self._db)

    def filter_by_access_level(self, user):
        return self.get_queryset().filter_by_access_level(user)

    def filter_by_metadata_access_level(self, user):
        return self.get_queryset().filter_by_metadata_access_level(user)



from django.db import models

from daiquiri.core.managers import AccessLevelManager
from daiquiri.jobs.managers import JobManager


class QueryJobManager(JobManager):

    def get_size(self, user):
        # get the size of all the tables of this user
        return self.filter_by_owner(user).exclude(phase=self.model.PHASE_ARCHIVED) \
                                         .aggregate(models.Sum('size'))['size__sum'] or 0


class ExampleManager(AccessLevelManager):

    pass



from django.conf import settings
from django.contrib.auth.models import Group
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from daiquiri.core.constants import ACCESS_LEVEL_CHOICES, ACCESS_LEVEL_PRIVATE
from daiquiri.core.managers import AccessLevelManager


class Schema(models.Model):

    order = models.IntegerField(
        default=0, null=True, blank=True,
        verbose_name=_('Order'),
        help_text=_('Position in lists.')
    )
    name = models.CharField(
        max_length=256,
        verbose_name=_('Name'),
        help_text=_('Name of the schema on the database server.')
    )
    title = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('Title'),
        help_text=_('Human readable title of the schema.')
    )
    description = models.TextField(
        blank=True, default='',
        verbose_name=_('Description'),
        help_text=_('A brief description of the schema to be displayed in the user interface.')
    )
    long_description = models.TextField(
        blank=True, default='',
        verbose_name=_('Long description'),
        help_text=_('A more extensive description of the schema to be displayed on the public schema page.')
    )
    attribution = models.TextField(
        blank=True, default='',
        verbose_name=_('Attribution'),
        help_text=_('The desired attribution for the schema.')
    )
    license = models.CharField(
        max_length=8, blank=True, default='',
        verbose_name=_('License')
    )
    doi = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('Digital object identifier')
    )
    related_identifiers = models.JSONField(
        blank=True, default=list,
        verbose_name=_('Related Identifiers'),
    )
    utype = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('IVOA Utype'),
    )
    creators = models.JSONField(
        blank=True, default=list,
        verbose_name=_('Creators'),
    )
    contributors = models.JSONField(
        blank=True, default=list,
        verbose_name=_('Contributors'),
    )
    published = models.DateField(
        null=True, blank=True,
        verbose_name=_('Published'),
    )
    updated = models.DateField(
        null=True, blank=True,
        verbose_name=_('Updated'),
    )
    access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES, default=ACCESS_LEVEL_PRIVATE,
        verbose_name=_('Access level')
    )
    metadata_access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES, default=ACCESS_LEVEL_PRIVATE,
        verbose_name=_('Metadata access level')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Groups'),
        help_text=_('The groups which have access to the schema.')
    )

    objects = AccessLevelManager()

    class Meta:
        ordering = ('order', 'name')

        verbose_name = _('Schema')
        verbose_name_plural = _('Schemas')

    def __str__(self):
        return self.name

    @property
    def query_strings(self):
        return [self.name]

    @property
    def license_label(self):
        return dict(settings.LICENSE_CHOICES)[self.license]

    @property
    def license_url(self):
        return settings.LICENSE_URLS[self.license]

    def admin_url(self):
        return reverse('admin:daiquiri_metadata_schema_change', args=[self.id])


class Table(models.Model):

    TYPE_TABLE = 'table'
    TYPE_VIEW = 'view'
    TYPE_CHOICES = (
        (TYPE_TABLE, _('table')),
        (TYPE_VIEW, _('view'))
    )

    schema = models.ForeignKey(
        Schema, related_name='tables', on_delete=models.CASCADE,
        verbose_name=_('Database'),
        help_text=_('Database the table belongs to.')
    )
    order = models.IntegerField(
        null=True, blank=True,
        verbose_name=_('Order'),
        help_text=_('Position in lists.')
    )
    name = models.CharField(
        max_length=256,
        verbose_name=_('Name'),
        help_text=_('Identifier of the table on the database server.')
    )
    title = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('Title'),
        help_text=_('Human readable title of the table.')
    )
    description = models.TextField(
        blank=True, default='',
        verbose_name=_('Description'),
        help_text=_('A brief description of the table to be displayed in the user interface.')
    )
    long_description = models.TextField(
        blank=True, default='',
        verbose_name=_('Long description'),
        help_text=_('A more extensive description of the table to be displayed on the public database page.')
    )
    attribution = models.TextField(
        blank=True, default='',
        verbose_name=_('Attribution'),
        help_text=_('The desired attribution for the table.')
    )
    license = models.CharField(
        max_length=8, blank=True, default='',
        verbose_name=_('License')
    )
    doi = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('Digital object identifier')
    )
    related_identifiers = models.JSONField(
        blank=True, default=list,
        verbose_name=_('Related Identifiers'),
    )
    type = models.CharField(
        max_length=8, choices=TYPE_CHOICES, default=TYPE_TABLE,
        verbose_name=_('Type of table')
    )
    nrows = models.BigIntegerField(
        null=True, blank=True,
        verbose_name=_('Number of rows in the table')
    )
    size = models.BigIntegerField(
        null=True, blank=True,
        verbose_name=_('Size of the table in bytes')
    )
    utype = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('IVOA Utype')
    )
    creators = models.JSONField(
        blank=True, default=list,
        verbose_name=_('Creators'),
    )
    contributors = models.JSONField(
        blank=True, default=list,
        verbose_name=_('Contributors'),
    )
    published = models.DateField(
        null=True, blank=True,
        verbose_name=_('Published'),
    )
    updated = models.DateField(
        null=True, blank=True,
        verbose_name=_('Updated'),
    )
    access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES, default=ACCESS_LEVEL_PRIVATE,
        verbose_name=_('Access level')
    )
    metadata_access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES, default=ACCESS_LEVEL_PRIVATE,
        verbose_name=_('Metadata access level')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Groups'),
        help_text=_('The groups which have access to the table.')
    )

    objects = AccessLevelManager()

    class Meta:
        ordering = ('schema__order', 'order', 'name')

        verbose_name = _('Table')
        verbose_name_plural = _('Tables')

    def __str__(self):
        return self.schema.name + '.' + self.name

    @property
    def query_strings(self):
        return [self.schema.name, self.name]

    @property
    def license_label(self):
        return dict(settings.LICENSE_CHOICES)[self.license]

    @property
    def license_url(self):
        return settings.LICENSE_URLS[self.license]

    @property
    def admin_url(self):
        return reverse('admin:daiquiri_metadata_table_change', args=[self.id])

    def discover(self, adapter):
        metadata = adapter.fetch_table(self.schema.name, self.name)
        try:
            self.type = metadata['type']
        except KeyError:
            # if the table does not exist in the database then do nothing
            return
        self.nrows = adapter.fetch_nrows(self.schema.name, self.name)
        self.size = adapter.fetch_size(self.schema.name, self.name)


class Column(models.Model):

    table = models.ForeignKey(
        Table, related_name='columns', on_delete=models.CASCADE,
        help_text=_('Table the column belongs to.')
    )
    order = models.IntegerField(
        null=True, blank=True,
        verbose_name=_('Order'),
        help_text=_('Position in lists.')
    )
    name = models.CharField(
        max_length=256,
        verbose_name=_('Name'),
        help_text=_('Identifier of the column on the database server.')
    )
    description = models.TextField(
        blank=True, default='',
        verbose_name=_('Description'),
        help_text=_('A brief description of the column to be displayed in the user interface.')
    )
    unit = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('Unit')
    )
    ucd = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('IVOA UCDs')
    )
    utype = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('IVOA Utype')
    )
    datatype = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('Datatype'),
        help_text=_('The datatype of the column on the database server.')
    )
    arraysize = models.IntegerField(
        null=True, blank=True,
        verbose_name=_('Arraysize'),
        help_text=_('The length of variable length datatypes, e.g. varchar(256).')
    )
    principal = models.BooleanField(
        default=False,
        verbose_name=_('Principal'),
        help_text=_('Designates whether the column is considered a core part of the content.')
    )
    indexed = models.BooleanField(
        default=False,
        verbose_name=_('Indexed'),
        help_text=_('Designates whether the column is indexed.')
    )
    std = models.BooleanField(
        default=False,
        verbose_name=_('Standard'),
        help_text=_('Designates whether the column is defined by some standard.')
    )
    index_for = models.CharField(
        max_length=256, blank=True, default='',
        verbose_name=_('Index for'),
        help_text=_('The columns which this column is an index for (e.g. for pgSphere).')
    )
    access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES, default=ACCESS_LEVEL_PRIVATE,
        verbose_name=_('Access level')
    )
    metadata_access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES, default=ACCESS_LEVEL_PRIVATE,
        verbose_name=_('Metadata access level')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Groups'),
        help_text=_('The groups which have access to the column.')
    )

    objects = AccessLevelManager()

    class Meta:
        ordering = ('table__schema__order', 'table__order', 'order', 'name')

        verbose_name = _('Column')
        verbose_name_plural = _('Columns')

    def __str__(self):
        return self.table.schema.name + '.' + self.table.name + '.' + self.name

    def get_width(self):
        return settings.METADATA_COLUMN_WIDTH.get(str(self), None) or \
            settings.METADATA_COLUMN_WIDTH.get('default', None)


    @property
    def query_strings(self):
        return [self.name]

    @property
    def indexed_columns(self):
        if self.index_for:
            return [
                (self.table.schema.name, self.table.name, name.strip()) for name in self.index_for.split(',')
            ] + [self.name]
        else:
            return None

    @property
    def admin_url(self):
        return reverse('admin:daiquiri_metadata_column_change', args=[self.id])

    def discover(self, adapter):
        metadata = adapter.fetch_column(self.table.schema.name, self.table.name, self.name)
        self.datatype = metadata.get('datatype', '')
        self.arraysize = metadata.get('arraysize', None)
        self.indexed = metadata.get('indexed', False)


class Function(models.Model):

    order = models.IntegerField(
        null=True, blank=True,
        verbose_name=_('Order'),
        help_text=_('Position in lists.')
    )
    name = models.CharField(
        max_length=256,
        verbose_name=_('Name'),
        help_text=_('Identifier of the function on the server.')
    )
    description = models.TextField(
        blank=True, default='',
        verbose_name=_('Description'),
        help_text=_('A brief description of the function to be displayed in the user interface.')
    )
    query_string = models.CharField(
        max_length=256,
        verbose_name=_('Query string'),
        help_text=_('Prototype of this function in a SQL query.')
    )
    access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES, default=ACCESS_LEVEL_PRIVATE,
        verbose_name=_('Access level')
    )
    metadata_access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES, default=ACCESS_LEVEL_PRIVATE,
        verbose_name=_('Metadata access level')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Groups'),
        help_text=_('The groups which have access to this function.')
    )

    objects = AccessLevelManager()

    class Meta:
        ordering = ('order', 'name')

        verbose_name = _('Function')
        verbose_name_plural = _('Functions')

    def __str__(self):
        return self.name

    @property
    def admin_url(self):
        return reverse('admin:daiquiri_metadata_function_change', args=[self.id])



import xml.etree.ElementTree as et
from unittest import mock

import pytest

from django.conf import settings
from django.test import override_settings
from django.urls import reverse
from django.utils.http import urlencode

import iso8601

from daiquiri.query.models import QueryJob

users = (
    ('admin', 'admin'),
    ('user', 'user'),
    ('test', 'test'),
    ('evil', 'evil'),
    ('anonymous', None),
)

url_names = {
    'list': 'tap:async-list',
    'detail': 'tap:async-detail',
    'results': 'tap:async-results',
    'result': 'tap:async-result',
    'parameters': 'tap:async-parameters',
    'destruction': 'tap:async-destruction',
    'executionduration': 'tap:async-executionduration',
    'phase': 'tap:async-phase',
    'error': 'tap:async-error',
    'quote': 'tap:async-quote',
    'owner': 'tap:async-owner'
}

instances = [
    '679b5ce8-bbcf-41c2-ba45-2340910bcfae',
    '5527f2b0-7130-466d-b8d2-dbf6c5f700bf',
    'c87926e4-8e5f-4032-a764-826b6a39c171',
    '1b6c93ef-4161-4402-b1a7-d1237657e807'
]

queries_public = [
    'SELECT ra, dec, parallax, id FROM daiquiri_data_obs.stars'
]

queries_internal = [
    'SELECT x, y, z, vx, vy, vz, id FROM daiquiri_data_sim.halos'
]

queries_private = [
    'SELECT id, "bool", "array", matrix FROM daiquiri_data_test.test'
]

uws_ns = '{http://www.ivoa.net/xml/UWS/v1.0}'


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
def test_get_job_list_xml(db, client, username, password):
    '''
    GET /{jobs} returns the job list as <uws:jobs> xml element. The archived
    jobs are not returned.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list'])
    response = client.get(url)
    assert response.status_code == 200, response.content

    if username == 'user':
        root = et.fromstring(response.content)

        assert root.tag == uws_ns + 'jobs'
        assert len(root) == 4
        for node in root:
            assert node.tag == uws_ns + 'jobref'


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
def test_get_job_list_xml_phase(db, client, username, password):
    '''
    GET /{jobs}?PHASE=<phase> returns the filtered joblist as <jobs>
    element.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list']) + '?PHASE=PENDING&PHASE=ERROR'
    response = client.get(url)
    assert response.status_code == 200, response.content

    if username == 'user':
        root = et.fromstring(response.content)
        assert len(root) == 2


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
def test_get_job_list_xml_after(db, client, username, password):
    '''
    GET /{jobs}?AFTER=2014-09-10T10:01:02.000 returns jobs with startTimes
    after the given [std:iso8601] time in UTC. The archived jobs are not
    returned.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list']) + '?AFTER=2017-01-01T01:00:00Z'
    response = client.get(url)
    assert response.status_code == 200, response.content

    if username == 'user':
        root = et.fromstring(response.content)
        assert len(root) == 2


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
def test_get_job_list_xml_last(db, client, username, password):
    '''
    GET /{jobs}?LAST=100 returns the given number of most recent jobs
    ordered by ascending startTimes. The archived jobs are not returned.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list']) + '?LAST=3'
    response = client.get(url)
    assert response.status_code == 200, response.content

    if username == 'user':
        root = et.fromstring(response.content)
        assert len(root) == 3


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_public)
def test_post_job_list_create_public(db, client, username, password, query):
    '''
    POST /{jobs} with an application/x-www-form-urlencoded set of KEY=VALUE
    creates a job with these parameters and redirects to /{jobs}/{job-id}
    as 303.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list'])
    response = client.post(url, urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query
    }), content_type='application/x-www-form-urlencoded')
    assert response.status_code == 303, response.content

    response = client.get(response.url + '/phase')
    assert response.status_code == 200, response.content
    assert response.content.decode() == 'PENDING'


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_internal)
def test_post_job_list_create_internal(db, client, username, password, query):
    '''
    POST /{jobs} with an application/x-www-form-urlencoded set of KEY=VALUE
    creates a job with these parameters and redirects to /{jobs}/{job-id}
    as 303.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list'])
    response = client.post(url, urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query
    }), content_type='application/x-www-form-urlencoded')

    if username != 'anonymous':
        assert response.status_code == 303, response.content

        response = client.get(response.url + '/phase')
        assert response.status_code == 200, response.content
        assert response.content.decode() == 'PENDING'
    else:
        assert response.status_code == 400, response.content


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_private)
def test_post_job_list_create_private(db, client, username, password, query):
    '''
    POST /{jobs} with an application/x-www-form-urlencoded set of KEY=VALUE
    creates a job with these parameters and redirects to /{jobs}/{job-id}
    as 303.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list'])
    response = client.post(url, urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query
    }), content_type='application/x-www-form-urlencoded')

    if username == 'test':
        assert response.status_code == 303, response.content

        response = client.get(response.url + '/phase')
        assert response.status_code == 200, response.content
        assert response.content.decode() == 'PENDING'
    else:
        assert response.status_code == 400, response.content


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_public)
def test_post_job_list_create_run(db, client, mocker, username, password, query):
    '''
    POST /{jobs} with an application/x-www-form-urlencoded set of
    KEY=VALUE and additionally PHASE=RUN to an non-existing {job-id} creates
    a job with these parameters and runs it.
    '''
    mocker.patch(settings.ADAPTER_DATABASE + '.submit_query', mock.Mock())
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_nrows', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_size', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_columns', mock.Mock(return_value=[]))
    mocker.patch(settings.ADAPTER_DATABASE + '.count_rows', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.create_user_schema_if_not_exists', mock.Mock())

    client.login(username=username, password=password)

    url = reverse(url_names['list'])
    response = client.post(url, urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query,
        'PHASE': 'RUN'
    }), content_type='application/x-www-form-urlencoded')
    assert response.status_code == 303, response.content

    response = client.get(response.url + '/phase')
    assert response.status_code == 200, response.content
    assert response.content.decode() == 'COMPLETED'


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_detail(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id} returns a job as <uws:job> xml element.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['detail'], kwargs={'pk': pk})
    response = client.get(url)

    if username == 'user':
        assert response.status_code == 200, response.content

        root = et.fromstring(response.content)
        assert root.tag == uws_ns + 'job'
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_results(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id}/results returns any results of the job {job-id} as
    <uws:results> xml element.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['results'], kwargs={'pk': pk})
    response = client.get(url)

    if username == 'user':
        assert response.status_code == 200, response.content

        root = et.fromstring(response.content)
        assert root.tag == uws_ns + 'results'
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_result(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id}/results/result returns a 303 to the stream url for this job.
    '''
    client.login(username=username, password=password)

    job = QueryJob.objects.get(pk=pk)
    url = reverse(url_names['result'], kwargs={
        'pk': pk,
        'result': 'result'
    })
    response = client.get(url)

    if username == 'user':
        if job.phase == 'COMPLETED':
            assert response.status_code == 200, response.content
        else:
            assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_parameters(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id}/parameters returns any parameters for the job
    {job-id} as <uws:parameters> xml element.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['parameters'], kwargs={'pk': pk})
    response = client.get(url)

    if username == 'user':
        assert response.status_code == 200, response.content

        root = et.fromstring(response.content)
        assert root.tag == uws_ns + 'parameters'
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_error(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id}/error returns any error message associated with
    {job-id} as text.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['error'], kwargs={'pk': pk})
    response = client.get(url)

    if username == 'user':
        assert response.status_code == 200, response.content
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_delete(db, client, username, password, pk):
    '''
    DELETE /{jobs}/{job-id} sets the job phase to ARCHIVED and deletes the
    results and redirects to /{jobs}/{job-id} as 303.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['detail'], kwargs={'pk': pk})
    response = client.delete(url)

    if username == 'user':
        job = QueryJob.objects.get(pk=pk)
        redirect_url = 'http://testserver' + reverse(url_names['detail'], kwargs={'pk': pk})

        assert response.status_code == 303
        assert response.url == redirect_url
        assert job.phase == 'ARCHIVED'
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_delete(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id} with ACTION=DELETE sets the job phase to ARCHIVED
    and deletes the results and redirects to /{jobs}/{job-id} as 303.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['detail'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'ACTION': 'DELETE'}), content_type='application/x-www-form-urlencoded')

    if username == 'user':
        job = QueryJob.objects.get(pk=pk)
        redirect_url = 'http://testserver' + reverse(url_names['detail'], kwargs={'pk': job.pk})

        assert response.status_code == 303
        assert response.url == redirect_url
        assert job.phase == 'ARCHIVED'
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_invalid(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id} with invalid ACTION returns 400.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['detail'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'ACTION': 'not_valid'}), content_type='application/x-www-form-urlencoded')

    if username == 'user':
        assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_missing(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id} without ACTION returns 400.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['detail'], kwargs={'pk': pk})
    response = client.post(url, content_type='application/x-www-form-urlencoded')

    if username == 'user':
        assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_destruction(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id}/destruction returns the destruction instant for
    {job-id} as [std:iso8601].
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['destruction'], kwargs={'pk': pk})
    response = client.get(url)

    if username == 'user':
        assert response.status_code == 200, response.content

        job = QueryJob.objects.get(pk=pk)
        if job.destruction_time:
            destruction_time = iso8601.parse_date(response.content.decode())
            assert destruction_time == job.destruction_time
        else:
            assert response.content.decode() == ''
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_destruction(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/destruction with DESTRUCTION={std:iso8601}
    (application/x-www-form-urlencoded) sets the destruction instant for
    {job-id} and redirects to /{jobs}/{job-id} as 303.
    '''
    client.login(username=username, password=password)

    destruction_time = '2016-01-01T00:00:00'

    url = reverse(url_names['destruction'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'DESTRUCTION': destruction_time}),
                           content_type='application/x-www-form-urlencoded')

    if username == 'user':
        job = QueryJob.objects.get(pk=pk)
        redirect_url = 'http://testserver' + reverse(url_names['detail'], kwargs={'pk': pk})

        assert response.status_code == 303
        assert response.url == redirect_url
        assert job.destruction_time == iso8601.parse_date(destruction_time)
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_destruction_invalid(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/destruction with invalid DESTRUCTION returns 400.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['destruction'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'DESTRUCTION': 'not_a_date'}),
                           content_type='application/x-www-form-urlencoded')

    if username == 'user':
        assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_destruction_missing(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/destruction without DESTRUCTION returns 400.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['destruction'], kwargs={'pk': pk})
    response = client.post(url, content_type='application/x-www-form-urlencoded')

    if username == 'user':
        assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_executionduration(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id}/executionduration returns the maximum execution
    duration of {job-id} as integer.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['executionduration'], kwargs={'pk': pk})
    response = client.get(url)

    if username == 'user':
        job = QueryJob.objects.get(pk=pk)
        assert response.status_code == 200, response.content
        assert int(response.content) == job.execution_duration
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_executionduration(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/executionduration with EXECUTIONDURATION={int}
    sets the maximum execution duration of {job-id} and redirects as to
    /{jobs}/{job-id} 303.
    '''
    execution_duration = 60
    client.login(username=username, password=password)

    url = reverse(url_names['executionduration'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'EXECUTIONDURATION': 60}), content_type='application/x-www-form-urlencoded')

    if username == 'user':
        job = QueryJob.objects.get(pk=pk)
        redirect_url = 'http://testserver' + reverse(url_names['detail'], kwargs={'pk': pk})

        assert response.status_code == 303
        assert response.url == redirect_url
        assert job.execution_duration == execution_duration
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_executionduration_invalid(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/executionduration with invalid EXECUTIONDURATION returns 400.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['executionduration'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'EXECUTIONDURATION': 'not_an_integer'}),
                           content_type='application/x-www-form-urlencoded')

    if username == 'user':
        assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_executionduration_missing(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/executionduration without EXECUTIONDURATION returns 400.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['executionduration'], kwargs={'pk': pk})
    response = client.post(url, content_type='application/x-www-form-urlencoded')

    if username == 'user':
        assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_phase(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id}/phase returns the phase of job {job-id} as one of
    the fixed strings.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['phase'], kwargs={'pk': pk})
    response = client.get(url)

    if username == 'user':
        job = QueryJob.objects.get(pk=pk)
        assert response.status_code == 200, response.content
        assert response.content == job.phase.encode()
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_phase_run(db, client, mocker, username, password, pk):
    '''
    POST /{jobs}/{job-id}/phase with PHASE=RUN runs the job {job-id} and
    redirects as to /{jobs}/{job-id} 303.
    '''
    mocker.patch(settings.ADAPTER_DATABASE + '.submit_query', mock.Mock())
    mocker.patch(settings.ADAPTER_DATABASE + '.fetch_size', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.count_rows', mock.Mock(return_value=100))
    mocker.patch(settings.ADAPTER_DATABASE + '.create_user_schema_if_not_exists', mock.Mock())

    client.login(username=username, password=password)

    job = QueryJob.objects.get(pk=pk)
    job_phase = job.phase
    url = reverse(url_names['phase'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'PHASE': 'RUN'}), content_type='application/x-www-form-urlencoded')

    if username == 'user':
        if job_phase in ['PENDING']:
            job.refresh_from_db()
            redirect_url = 'http://testserver' + reverse(url_names['detail'], kwargs={'pk': pk})

            assert response.status_code == 303
            assert response.url == redirect_url
            assert job.phase == 'COMPLETED'
        else:
            assert response.status_code == 400

    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_phase_abort(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/phase with PHASE=ABORT aborts the job {job-id} and
    redirects as to /{jobs}/{job-id} 303.
    '''
    client.login(username=username, password=password)

    job = QueryJob.objects.get(pk=pk)
    job_phase = job.phase
    url = reverse(url_names['phase'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'PHASE': 'ABORT'}), content_type='application/x-www-form-urlencoded')

    if username == 'user':
        job.refresh_from_db()
        redirect_url = 'http://testserver' + reverse(url_names['detail'], kwargs={'pk': pk})

        assert response.status_code == 303
        assert response.url == redirect_url

        if job_phase in ['QUEUED', 'EXECUTING']:
            assert job.phase == 'ABORTED'
        else:
            assert job.phase == job_phase
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_phase_invalid(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/phase with invalid PHASE returns 400.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['phase'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'PHASE': 'invalid'}), content_type='application/x-www-form-urlencoded')

    if username == 'user':
        assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_phase_missing(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/phase without PHASE returns 400.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['phase'], kwargs={'pk': pk})
    response = client.post(url, content_type='application/x-www-form-urlencoded')

    if username == 'user':
        assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_post_job_phase_unsupported(db, client, username, password, pk):
    '''
    POST /{jobs}/{job-id}/phase with PHASE=unsupported returns 400.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['phase'], kwargs={'pk': pk})
    response = client.post(url, urlencode({'PHASE': 'unsupported'}), content_type='application/x-www-form-urlencoded')

    if username == 'user':
        assert response.status_code == 400
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_quote(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id}/quote returns the quote for {job-id} as [std:iso8601].
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['quote'], kwargs={'pk': pk})
    response = client.get(url)

    if username == 'user':
        assert response.status_code == 200, response.content
        assert response.content.decode() == ''
    else:
        assert response.status_code == 404


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_get_job_owner(db, client, username, password, pk):
    '''
    GET /{jobs}/{job-id}/owner returns the owner of the job {job-id} as an
    appropriate identifier.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['owner'], kwargs={'pk': pk})
    response = client.get(url)

    if username == 'user':
        job = QueryJob.objects.get(pk=pk)
        assert response.status_code == 200, response.content
        assert response.content.decode() == job.owner.username
    else:
        assert response.status_code == 404



from django.template.loader import TemplateDoesNotExist, get_template

from rest_framework import serializers

from daiquiri.jobs.serializers import AsyncJobSerializer, SyncJobSerializer

from .models import Example, QueryJob
from .utils import get_query_form, get_query_form_adapter
from .validators import TableNameValidator, UploadFileValidator, UploadParamValidator


class FormListSerializer(serializers.Serializer):

    key = serializers.CharField()
    label = serializers.CharField()

    # TODO: remove
    form_service = serializers.SerializerMethodField()

    def get_form_service(self, obj):
        return obj['key'][0].upper() + obj['key'][1:] + 'FormService'


class FormDetailSerializer(serializers.Serializer):

    key = serializers.CharField()
    label = serializers.CharField()
    template = serializers.SerializerMethodField(required=False)
    fields = serializers.SerializerMethodField(method_name='get_adapter_fields')
    submit = serializers.CharField(default=None)

    def get_template(self, obj):
        try:
            return get_template(obj['template']).render(request=self.context.get('request')).strip()
        except (KeyError, TemplateDoesNotExist):
            return None

    def get_adapter_fields(self, obj):
        adapter = get_query_form_adapter(obj)
        return adapter.get_fields() if adapter else None


class DropdownSerializer(serializers.Serializer):

    key = serializers.CharField()
    label = serializers.CharField()
    classes = serializers.CharField(default=None)
    options = serializers.JSONField(default=None)


class DownloadSerializer(serializers.Serializer):

    key = serializers.CharField()
    form = serializers.JSONField(default=None)


class QueryJobSerializer(serializers.ModelSerializer):

    class Meta:
        model = QueryJob
        fields = (
            'id',
        )


class QueryJobIndexSerializer(serializers.ModelSerializer):

    class Meta:
        model = QueryJob
        fields = (
            'id',
            'table_name',
            'run_id',
            'phase'
        )


class QueryJobRetrieveSerializer(serializers.ModelSerializer):

    sources = serializers.SerializerMethodField()
    columns = serializers.SerializerMethodField()

    class Meta:
        model = QueryJob
        fields = (
            'id',
            'run_id',
            'phase',
            'phase_label',
            'creation_time',
            'creation_time_label',
            'start_time',
            'start_time_label',
            'end_time',
            'end_time_label',
            'execution_duration',
            'time_queue',
            'time_query',
            'destruction_time',
            'error_summary',
            'job_type',
            'schema_name',
            'table_name',
            'query_language',
            'query_language_label',
            'query',
            'native_query',
            'actual_query',
            'queue',
            'nrows',
            'size',
            'sources',
            'columns'
        )

    def get_sources(self, obj):
        if obj.metadata:
            return obj.metadata.get('sources', [])
        else:
            return []

    def get_columns(self, obj):
        if obj.metadata:
            return obj.metadata.get('columns', [])
        else:
            return []


class QueryJobCreateSerializer(serializers.ModelSerializer):

    table_name = serializers.CharField(required=False, allow_blank=True, max_length=256,
                                       validators=[TableNameValidator()])
    queue = serializers.CharField(required=False)
    query_language = serializers.CharField(required=True)
    query = serializers.CharField(required=True)
    run_id = serializers.CharField(default='', allow_blank=True)

    class Meta:
        model = QueryJob
        fields = (
            'id',
            'table_name',
            'queue',
            'query_language',
            'query',
            'run_id'
        )


class QueryJobFormSerializer(QueryJobCreateSerializer):

    query_language = serializers.SerializerMethodField()
    query = serializers.SerializerMethodField()

    class Meta:
        model = QueryJob
        fields = QueryJobCreateSerializer.Meta.fields

    def __init__(self, *args, **kwargs):
        form_key = kwargs.pop('form_key')
        form = get_query_form(form_key)

        self.adapter = get_query_form_adapter(form)

        super().__init__(*args, **kwargs)

        for field in self.adapter.get_fields():
            if field.get('type') == 'number':
                self.fields[field['key']] = serializers.FloatField(required=True)
            else:
                self.fields[field['key']] = serializers.CharField(required=False)

    def get_query_language(self, obj):
        return self.adapter.get_query_language(obj)

    def get_query(self, obj):
        return self.adapter.get_query(obj)


class QueryJobUpdateSerializer(serializers.ModelSerializer):

    table_name = serializers.CharField(required=True, validators=[TableNameValidator()])

    class Meta:
        model = QueryJob
        fields = (
            'id',
            'table_name',
            'run_id'
        )


class QueryJobUploadSerializer(serializers.ModelSerializer):

    table_name = serializers.CharField(required=False, validators=[TableNameValidator()])
    run_id = serializers.CharField(default='')
    file = serializers.FileField(max_length=None, allow_empty_file=False, validators=[UploadFileValidator()])

    class Meta:
        model = QueryJob
        fields = (
            'table_name',
            'run_id',
            'file'
        )


class QueryLanguageSerializer(serializers.Serializer):

    id = serializers.SerializerMethodField(required=False)
    text = serializers.CharField(source='label')
    quote_char = serializers.CharField()

    def get_id(self, obj):
        return '{key}-{version}'.format(**obj)



class QueryDownloadFormatSerializer(serializers.Serializer):

    key = serializers.CharField()
    extension = serializers.CharField()
    content_type = serializers.CharField()
    label = serializers.CharField()
    help = serializers.CharField()



class ExampleSerializer(serializers.ModelSerializer):

    class Meta:
        model = Example
        fields = (
            'id',
            'order',
            'name',
            'description',
            'query_language',
            'query_string',
            'access_level',
            'groups'
        )


class UserExampleSerializer(serializers.ModelSerializer):

    class Meta:
        model = Example
        fields = (
            'id',
            'order',
            'name',
            'description',
            'query_string',
            'query_language',
            'groups'
        )


class SyncQueryJobSerializer(SyncJobSerializer):

    RESPONSEFORMAT = serializers.CharField(required=False)
    FORMAT = serializers.CharField(required=False)

    TABLE_NAME = serializers.CharField(required=False, max_length=256, validators=[TableNameValidator()])
    LANG = serializers.CharField(required=True)
    QUERY = serializers.CharField(required=True)

    UPLOAD = serializers.CharField(required=False, default='', validators=[UploadParamValidator()])


class AsyncQueryJobSerializer(AsyncJobSerializer):

    RESPONSEFORMAT = serializers.CharField(required=False)
    FORMAT = serializers.CharField(required=False)

    TABLE_NAME = serializers.CharField(required=False, max_length=256, validators=[TableNameValidator()])
    QUEUE = serializers.CharField(required=False)
    LANG = serializers.CharField(required=True)
    QUERY = serializers.CharField(required=True)

    UPLOAD = serializers.CharField(required=False, default='', validators=[UploadParamValidator()])



import os
import sys

from django.conf import settings
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

import requests
from astropy.io.votable import parse_single_table

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.utils import handle_file_upload, human2bytes, import_class
from daiquiri.metadata.models import Column, Table


def get_download_config(download_key):
    try:
        return next(filter(lambda c: c['key'] == download_key, settings.QUERY_DOWNLOADS))
    except StopIteration:
        return None


def get_format_config(format_key):

    for format_config in settings.QUERY_DOWNLOAD_FORMATS:
        if format_config['key'] == format_key:
            return format_config

    return None


def get_default_table_name():
    return now().strftime("%Y-%m-%d-%H-%M-%S-%f")


def get_user_schema_name(user):
    if not user or user.is_anonymous:
        username = 'anonymous'
    else:
        username = user.username

    return settings.QUERY_USER_SCHEMA_PREFIX + username


def get_user_upload_directory(user):
    if not user or user.is_anonymous:
        username = 'anonymous'
    else:
        username = user.username

    return os.path.join(settings.QUERY_UPLOAD_DIR, username)


def get_quota(user, quota_settings='QUERY_QUOTA'):

    quota_config = getattr(settings, quota_settings)

    if not user or user.is_anonymous:
        quota = human2bytes(quota_config.get('anonymous'))

    else:
        quota = human2bytes(quota_config.get('user'))

        # apply quota for user
        users = quota_config.get('users')
        if users:
            user_quota = human2bytes(users.get(user.username))
            quota = user_quota if user_quota > quota else quota

        # apply quota for group
        groups = quota_config.get('groups')
        if groups:
            for group in user.groups.all():
                group_quota = human2bytes(groups.get(group.name))
                quota = group_quota if group_quota > quota else quota

    return quota


def get_max_active_jobs(user):
    if not user or user.is_anonymous:
        count = int(settings.QUERY_MAX_ACTIVE_JOBS.get('anonymous') or 0)

    else:
        count = int(settings.QUERY_MAX_ACTIVE_JOBS.get('user') or 0)

        # apply quota for user
        users = int(settings.QUERY_MAX_ACTIVE_JOBS.get('users') or 0)
        if users:
            user_count = int(users.get(user.username))
            count = user_count if user_count and user_count > count else count

        # apply quota for group
        groups = int(settings.QUERY_MAX_ACTIVE_JOBS.get('groups') or 0)
        if groups:
            for group in user.groups.all():
                group_count = int(groups.get(group.name))
                count = group_count if group_count and group_count > count else count

    return count


def fetch_user_schema_metadata(user, jobs):

    schema_name = get_user_schema_name(user)

    schema = {
        'order': sys.maxsize,
        'name': schema_name,
        'query_strings': [schema_name],
        'description': _('Your personal schema'),
        'tables': []
    }

    for job in jobs:
        table = {
            'name': job.table_name,
            'query_strings': [schema_name, job.table_name]
        }

        if job.metadata:
            table['columns'] = job.metadata.get('columns', {})

            for column in table['columns']:
                column['query_strings'] = [column['name']]

        schema['tables'].append(table)

    return [schema]


def get_indexed_objects():
    indexed_objects = {}

    for column in Column.objects.exclude(index_for=''):
        # TODO implement xtype 'spoint' properly

        #if column.datatype not in indexed_objects:
        #    indexed_objects[column.datatype] = [column.indexed_columns]
        #else:
        #    indexed_objects[column.datatype].append(column.indexed_columns)

        if 'spoint' not in indexed_objects:
            indexed_objects['spoint'] = [column.indexed_columns]
        else:
            indexed_objects['spoint'].append(column.indexed_columns)

    return indexed_objects


def get_job_sources(job):
    sources = []

    if 'tables' in job.metadata:
        for schema_name, table_name in job.metadata['tables']:
            table = {
                'schema_name': schema_name,
                'table_name': table_name
            }

            # fetch additional metadata from the metadata store
            try:
                original_table = Table.objects.get(
                    name=table_name,
                    schema__name=schema_name
                )

                if settings.METADATA_BASE_URL:
                    metadata_url = '{}/{}/{}'.format(settings.METADATA_BASE_URL.strip('/'), schema_name, table_name)
                else:
                    metadata_url = ''

                table.update({
                    'title': original_table.title,
                    'description': original_table.description,
                    'attribution': original_table.attribution,
                    'license': original_table.license,
                    'doi': original_table.doi,
                    'url': metadata_url
                })

                sources.append(table)

            except Table.DoesNotExist:
                pass

    return sources


def get_job_column(job, display_column_name):
    try:
        schema_name, table_name, column_name = \
            job.metadata['display_columns'][display_column_name]
    except (ValueError, KeyError):
        return {}

    if (schema_name == settings.TAP_UPLOAD) or (schema_name == get_user_schema_name(job.owner)):
        # for TAP_UPLOAD get the information directly from the database
        column = DatabaseAdapter().fetch_column(schema_name, table_name, column_name)

        # add the uploads metadata (ucds, units, description)
        for upload_column in job.metadata['upload_columns']:
            if upload_column['name'] == column['name']:
                column.update(upload_column)
                break

        # add the job metadata (ucds, units, description)
        for user_column in job.metadata['user_columns']:
            if user_column['name'] == column['name']:
                column.update(user_column)

        return column

    else:
        # for regular schemas consult the metadata store
        try:
            column = Column.objects.get(
                name=column_name,
                table__name=table_name,
                table__schema__name=schema_name
            )

            return {
                'name': column.name,
                'description': column.description,
                'unit': column.unit,
                'ucd': column.ucd,
                'utype': column.utype,
                'datatype': column.datatype,
                'arraysize': column.arraysize,
                'principal': column.principal,
                'indexed': False,
                'std': column.std
            }

        except Column.DoesNotExist:
            return {}


def get_job_columns(job):
    columns = []

    if job.phase == job.PHASE_COMPLETED or job.job_type == job.JOB_TYPE_SYNC:
        database_columns = DatabaseAdapter().fetch_columns(job.schema_name, job.table_name)

        for database_column in database_columns:
            column = get_job_column(job, database_column['name'])
            column.update(database_column)
            columns.append(column)

    return columns


def handle_upload_param(request, upload_param):
    if upload_param:
        uploads = {}
        for upload in upload_param.split(';'):
            resource_name, uri = upload.split(',')

            if uri.startswith('param:'):
                file_field = uri[len('param:'):]
                file_path = handle_file_upload(get_user_upload_directory(request.user), request.data[file_field])
                uploads[resource_name] = file_path

            else:
                uploads[resource_name] = uri

        return uploads
    else:
        return {}


def ingest_uploads(uploads, user):

    columns = []

    if uploads:
        for table_name, location in uploads.items():
            if location.startswith('http:') or location.startswith('https:'):
                file_path = fetch_file(user, location)
            else:
                file_path = location

            columns = ingest_table(settings.TAP_UPLOAD, table_name, file_path, drop_table=True)

    return columns


def fetch_file(user, url):
    response = requests.get(url, allow_redirects=True)

    file_path = os.path.join(get_user_upload_directory(user), url.split('/')[-1])
    with open(file_path, 'wb') as f:
        f.write(response.content)

    return file_path


def catch_special_types(field):
    '''Catch any DB specific datatype, which are not VO compatible. i.e.: like spoint for postgres.
    '''
    adapter = DatabaseAdapter()

    if adapter.database_config['ENGINE'] == 'django.db.backends.postgresql':

        if( field.name == "pos" and field.datatype in ("char", "unicodeChar") ):
            datatype = "spoint"
            arraysize = None
        else:
            datatype = field.datatype
            arraysize = field.arraysize

    else:
        datatype = field.datatype
        arraysize = field.arraysize

    return(datatype, arraysize)


def ingest_table(schema_name, table_name, file_path, drop_table=False):
    adapter = DatabaseAdapter()

    table = parse_single_table(file_path, pedantic=False)

    columns = []
    for field in table.fields:

        datatype, arraysize = catch_special_types(field)

        columns.append({
            'name': field.name,
            'datatype': datatype,
            'ucd': field.ucd,
            'unit': str(field.unit),
            'arraysize': arraysize,
        })

    if drop_table:
        adapter.drop_table(schema_name, table_name)

    adapter.create_table(schema_name, table_name, columns)
    adapter.insert_rows(schema_name, table_name, columns, table.array, table.array.mask)

    os.remove(file_path)

    return columns


def get_query_form(form_key):
    try:
        return next(form for form in settings.QUERY_FORMS if form.get('key') == form_key)
    except StopIteration as e:
        raise RuntimeError(f'Query form "{form_key}"" not found.') from e


def get_query_form_adapter(form):
    adapter_class = form.get('adapter')
    if adapter_class:
        try:
            return import_class(form['adapter'])()
        except AttributeError as e:
            raise RuntimeError(f'Query form adapter "{adapter_class}"" could not be imported.') from e


def get_query_language_choices():
    return [
        (f"{query_language['key']}-{query_language['version']}", query_language['label']) \
        for query_language in settings.QUERY_LANGUAGES
    ]


def get_query_language_label(query_language):
    return next(iter(
        ql['label']
        for ql in settings.QUERY_LANGUAGES
        if query_language in [ql['key'], '{key}-{version}'.format(**ql)]
    ), None)


def get_queue_choices():
    return [
        (queue['key'], queue['label']) for queue in settings.QUERY_QUEUES
    ]


def get_query_download_format_choices():
    return [
        (download_format['key'], download_format['label']) for download_format in settings.QUERY_DOWNLOAD_FORMATS
    ]



from django.contrib.auth.models import User
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('username', help='Username of the user to activate.')
        parser.add_argument('--deactivate', action='store_true', default=False, help='Deactivate instead of activate.')

    def handle(self, *args, **options):
        is_active = False if options['deactivate'] else True

        user = User.objects.get(username=options['username'])
        user.is_active = is_active
        user.save()



# Generated by Django 4.2.19 on 2025-03-03 13:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0016_django2'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='job_type',
            field=models.CharField(choices=[('SYNC', 'Synchronous'), ('ASYNC', 'Asynchronous'), ('INTERFACE', 'Interface')], max_length=10),
        ),
    ]



import pytest

from django.urls import reverse

from ..models import ContactMessage

users = (
    ('admin', 'admin'),
    ('manager', 'manager'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'list': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'detail': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'create': {
        'admin': 201, 'manager': 201, 'user': 403, 'anonymous': 403
    },
    'update': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'delete': {
        'admin': 204, 'manager': 204, 'user': 403, 'anonymous': 403
    }
}

urlnames = {
    'list': 'contact:message-list',
    'detail': 'contact:message-detail'
}


instances = [1, 2]


@pytest.mark.parametrize(('username', 'password'), users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == status_map['list'][username], response.json()

    if response.status_code == 200:
        assert response.json().get('count') == 2
        assert len(response.json().get('results')) == 2


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_detail(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = ContactMessage.objects.get(pk=pk)
    url = reverse(urlnames['detail'], args=[pk])
    response = client.get(url)
    assert response.status_code == status_map['detail'][username], response.json()

    if response.status_code == 200:
        assert response.json().get('email') == instance.email


@pytest.mark.parametrize(('username', 'password'), users)
def test_create(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url, {
        'author': 'Tanja Test',
        'email': 'test@example.com',
        'status': 'ACTIVE',
        'subject': 'Test',
        'message': 'This is a test.'
    })
    assert response.status_code == status_map['create'][username], response.json()


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_update(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = ContactMessage.objects.get(pk=pk)
    instance_author = instance.author
    url = reverse(urlnames['detail'], args=[pk])
    response = client.put(url, {
        'author': 'Tanja Test',
        'email': 'test@example.com',
        'status': 'ACTIVE',
        'subject': 'Test',
        'message': 'This is a test.'
    }, content_type='application/json')
    assert response.status_code == status_map['update'][username], response.json()

    instance.refresh_from_db()
    assert instance.author == ('Tanja Test' if response.status_code == 200 else instance_author)


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_delete(db, client, username, password, pk):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[pk])
    response = client.delete(url)
    assert response.status_code == status_map['delete'][username], response.json()
    assert ContactMessage.objects.filter(pk=pk).exists() is not (response.status_code == 204)



from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import gettext_lazy as _

from daiquiri.core.encoders import UUIDJSONEncoder


class Record(models.Model):

    time = models.DateTimeField()

    resource_type = models.CharField(max_length=32)
    resource = models.JSONField(encoder=UUIDJSONEncoder)

    client_ip = models.GenericIPAddressField(blank=True, null=True)
    user = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL)

    size = models.PositiveBigIntegerField(null=True, blank=True)

    class Meta:
        ordering = ('-time', )

        verbose_name = _('Record')
        verbose_name_plural = _('Records')

    def __str__(self):
        return f'{self.time} {self.resource_type}'



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-03-31 11:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0003_owner_fk'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='job_type',
            field=models.CharField(choices=[('QUERY', 'Query')], max_length=10),
        ),
        migrations.AlterField(
            model_name='job',
            name='phase',
            field=models.CharField(choices=[('PENDING', 'Pending'), ('QUEUED', 'Queued'), ('EXECUTING', 'Executing'), ('COMPLETED', 'Completed'), ('ERROR', 'Error'), ('ABORTED', 'Aborted'), ('UNKNOWN', 'Unknown'), ('HELD', 'Held'), ('SUSPENDED', 'Suspended'), ('ARCHIVED', 'Archived')], max_length=10),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-25 11:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0009_arraysize'),
    ]

    operations = [
        migrations.AlterField(
            model_name='column',
            name='arraysize',
            field=models.IntegerField(blank=True, help_text='The length of variable length datatypes, e.g. varchar(256).', null=True, verbose_name='Arraysize'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-22 09:53
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_stats', '0003_data_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='record',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 3.2.12 on 2022-03-21 14:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0026_use_django_jsonfield'),
    ]

    operations = [
        migrations.RenameField(
            model_name='downloadjob',
            old_name='job',
            new_name='query_job',
        ),
        migrations.RenameField(
            model_name='queryarchivejob',
            old_name='job',
            new_name='query_job',
        ),
    ]



# Generated by Django 2.2 on 2019-06-20 14:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_oai', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='record',
            name='resource_id',
            field=models.PositiveIntegerField(db_index=True, null=True, verbose_name='Resource id'),
        ),
        migrations.AddField(
            model_name='record',
            name='resource_type',
            field=models.CharField(db_index=True, default='', max_length=32, verbose_name='Resource type'),
            preserve_default=False,
        ),
    ]



TAP_SCHEMA_METADATA = {
    'order': 1000,
    'description': 'The TAP SCHEMA of the service',
    'access_level': 'PUBLIC',
    'metadata_access_level': 'PUBLIC',
    'tables': [
        {
            'name': 'schemas',
            'description': 'The table of schemas for the service',
            'order': 1,
            'access_level': 'PUBLIC',
            'metadata_access_level': 'PUBLIC',
            'columns': [
                {
                    'name': 'id',
                    'description': 'Internal ID of the schema',
                    'order': 1,
                    'datatype': 'integer',
                    'arraysize': None,
                    'std': False,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'schema_name',
                    'description': 'The name of the schema',
                    'order': 2,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'utype',
                    'description': 'The IVOA UTYPE of the schema',
                    'order': 3,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'description',
                    'description': 'The description of the schema',
                    'order': 4,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                }
            ]
        },
        {
            'name': 'tables',
            'description': 'The table of tables for the service',
            'order': 2,
            'access_level': 'PUBLIC',
            'metadata_access_level': 'PUBLIC',
            'columns': [
                {
                    'name': 'id',
                    'description': 'Internal ID of the table',
                    'order': 1,
                    'datatype': 'integer',
                    'arraysize': None,
                    'std': False,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'schema_name',
                    'description': 'The name of the schema of the table',
                    'order': 2,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'table_name',
                    'description': 'The name of the table',
                    'order': 3,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'table_type',
                    'description': 'The type of the table (table or view)',
                    'order': 4,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'utype',
                    'description': 'The IVOA UTYPE of the table',
                    'order': 5,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'description',
                    'description': 'The description of the table',
                    'order': 6,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'table_index',
                    'description': 'The ordering index of the table',
                    'order': 7,
                    'datatype': 'integer',
                    'arraysize': None,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'schema_id',
                    'description': 'The foreign key to the schema of this table',
                    'order': 8,
                    'datatype': 'integer',
                    'arraysize': None,
                    'std': False,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                }
            ]
        },
        {
            'name': 'columns',
            'description': 'The table of columns of the service',
            'order': 3,
            'access_level': 'PUBLIC',
            'metadata_access_level': 'PUBLIC',
            'columns': [
                {
                    'name': 'id',
                    'description': 'Internal ID of the column',
                    'order': 1,
                    'datatype': 'integer',
                    'arraysize': None,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'table_name',
                    'description': 'The name of the table of the column',
                    'order': 2,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'column_name',
                    'description': 'The name of the column',
                    'order': 3,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'datatype',
                    'description': 'The datatype of the column',
                    'order': 4,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'arraysize',
                    'description': 'The size of the column for variable length datatypes',
                    'order': 5,
                    'datatype': 'integer',
                    'arraysize': None,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'size',
                    'description': 'The size of the column for variable length datatypes (legacy)',
                    'order': 6,
                    'datatype': 'integer',
                    'arraysize': None,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'description',
                    'description': 'The description of the column',
                    'order': 7,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'utype',
                    'description': 'The IVOA UTYPE of the column',
                    'order': 8,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'unit',
                    'description': 'The Unit of the column',
                    'order': 9,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'ucd',
                    'description': 'The IVOA UCD of the column',
                    'order': 10,
                    'datatype': 'char',
                    'arraysize': 256,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'principal',
                    'description': 'Designates if the column is considered a core part of the service',
                    'order': 11,
                    'datatype': 'boolean',
                    'arraysize': None,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'indexed',
                    'description': 'Designates if the column is indexed',
                    'order': 12,
                    'datatype': 'boolean',
                    'arraysize': None,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'std',
                    'description': 'Designates if the column is defined by some standard',
                    'order': 13,
                    'datatype': 'boolean',
                    'arraysize': None,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'column_index',
                    'description': 'The ordering index of the column',
                    'order': 14,
                    'datatype': 'integer',
                    'arraysize': None,
                    'std': True,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                },
                {
                    'name': 'table_id',
                    'description': 'The foreign key to the table of this column',
                    'order': 15,
                    'datatype': 'integer',
                    'arraysize': None,
                    'std': False,
                    'access_level': 'PUBLIC',
                    'metadata_access_level': 'PUBLIC'
                }
            ]
        }
    ]
}
